@@ -1,7 +1,7 @@
 /* Trampoline construction */
 
 /*
- * Copyright 1995-1999, 2001 Bruno Haible, <haible@clisp.cons.org>
+ * Copyright 1995-1999, 2001-2002 Bruno Haible, <haible@clisp.cons.org>
  *
  * This is free software distributed under the GNU General Public Licence
  * described in the file COPYING. Contact the author if you don't have this
@@ -21,12 +21,16 @@
 #endif
 #endif
 #if defined(__rs6000__)
+#if defined(__NetBSD__)
+#define __powerpcnetbsd__
+#else
 #if !defined(_AIX)
 #define __rs6000sysv4__  /* SysV.4 ABI, real machine code. */
 #else
 #define __rs6000aix__  /* AIX ABI, just a closure. */
 #endif
 #endif
+#endif
 #if defined(__hppanew__)
 /*
  * A function pointer is a biased pointer to a data area whose first word
@@ -100,14 +104,18 @@ extern void (*tramp_r) (); /* trampoline prototype */
 
 /* Declare malloc(), free(). */
 #ifndef malloc
-#if defined(__STDC__) || defined(__cplusplus)
+#ifdef __cplusplus
+extern "C" RETMALLOCTYPE malloc (MALLOC_SIZE_T size);
+#elif defined(__STDC__)
 extern RETMALLOCTYPE malloc (MALLOC_SIZE_T size);
 #else
 extern RETMALLOCTYPE malloc();
 #endif
 #endif
 #ifndef free
-#if defined(__STDC__) || defined(__cplusplus)
+#ifdef __cplusplus
+extern "C" RETFREETYPE free (RETMALLOCTYPE ptr);
+#elif defined(__STDC__)
 extern RETFREETYPE free (RETMALLOCTYPE ptr);
 #else
 extern RETFREETYPE free();
@@ -115,7 +123,9 @@ extern RETFREETYPE free();
 #endif
 
 /* Declare abort(). */
-#if defined(__STDC__) || defined(__cplusplus)
+#ifdef __cplusplus
+extern "C" ABORT_VOLATILE RETABORTTYPE abort (void);
+#elif defined(__STDC__)
 extern ABORT_VOLATILE RETABORTTYPE abort (void);
 #else
 extern ABORT_VOLATILE RETABORTTYPE abort();
@@ -123,7 +133,9 @@ extern ABORT_VOLATILE RETABORTTYPE abort();
 
 /* Declare getpagesize(). */
 #ifdef HAVE_GETPAGESIZE
-#if defined(__STDC__) || defined(__cplusplus)
+#ifdef __cplusplus
+extern "C" RETGETPAGESIZETYPE getpagesize (void);
+#elif defined(__STDC__)
 extern RETGETPAGESIZETYPE getpagesize (void);
 #else
 extern RETGETPAGESIZETYPE getpagesize ();
@@ -158,7 +170,9 @@ extern RETGETPAGESIZETYPE getpagesize ();
 #include <sys/types.h>
 #include <sys/mman.h>
 #if !defined(__convex__)
-#if defined(__STDC__) || defined(__cplusplus)
+#ifdef __cplusplus
+extern "C" int mprotect (MPROTECT_CONST MMAP_ADDR_T addr, MMAP_SIZE_T len, int prot);
+#elif defined(__STDC__)
 extern int mprotect (MPROTECT_CONST MMAP_ADDR_T addr, MMAP_SIZE_T len, int prot);
 #else
 extern int mprotect ();
@@ -175,7 +189,9 @@ extern int mprotect ();
 #if !defined(PROT_EXEC) && defined(PROT_EXECUTE) /* Irix 4.0.5 needs this */
 #define PROT_EXEC PROT_EXECUTE
 #endif
-#if defined(__STDC__) || defined(__cplusplus)
+#ifdef __cplusplus
+extern "C" RETMMAPTYPE mmap (MMAP_ADDR_T addr, MMAP_SIZE_T len, int prot, int flags, int fd, off_t off);
+#elif defined(__STDC__)
 extern RETMMAPTYPE mmap (MMAP_ADDR_T addr, MMAP_SIZE_T len, int prot, int flags, int fd, off_t off);
 #else
 extern RETMMAPTYPE mmap ();
@@ -190,7 +206,13 @@ extern RETMMAPTYPE mmap ();
 #ifdef OPEN_NEEDS_SYS_FILE_H
 #include <sys/file.h>
 #endif
-#if defined(__STDC__) || defined(__cplusplus)
+#ifdef __cplusplus
+#ifdef OPEN_DOTS
+extern "C" int open (OPEN_CONST char* path, int flags, ...);
+#else
+extern "C" int open (OPEN_CONST char* path, int flags, mode_t mode);
+#endif
+#elif defined(__STDC__)
 #ifdef OPEN_DOTS
 extern int open (OPEN_CONST char* path, int flags, ...);
 #else
@@ -209,7 +231,15 @@ extern int open ();
 #ifdef HAVE_SYS_SYSMACROS_H
 #include <sys/sysmacros.h>
 #endif
-#if defined(__STDC__) || defined(__cplusplus)
+#ifdef __cplusplus
+extern "C" int shmget (key_t key, SHMGET_SIZE_T size, int shmflg);
+extern "C" RETSHMATTYPE shmat (int shmid, SHMAT_CONST RETSHMATTYPE shmaddr, int shmflg);
+#ifdef SHMCTL_DOTS
+extern "C" int shmctl (int shmid, int cmd, ...);
+#else
+extern "C" int shmctl (int shmid, int cmd, struct shmid_ds * buf);
+#endif
+#elif defined(__STDC__)
 extern int shmget (key_t key, SHMGET_SIZE_T size, int shmflg);
 extern RETSHMATTYPE shmat (int shmid, SHMAT_CONST RETSHMATTYPE shmaddr, int shmflg);
 #ifdef SHMCTL_DOTS
@@ -259,7 +289,7 @@ extern int shmctl ();
 #include <sys/syslocal.h>
 #endif
 /* Inline assembly function for instruction cache flush. */
-#if defined(__sparc__) || defined(__sparc64__) || defined(__alpha__) || defined(__hppaold__) || defined(__rs6000sysv4__) || defined(__convex__)
+#if defined(__sparc__) || defined(__sparc64__) || defined(__alpha__) || defined(__hppaold__) || defined(__rs6000sysv4__) || defined(__convex__) || defined(__powerpcnetbsd__)
 #ifdef __GNUC__
 extern inline
 #if defined(__sparc__) || defined(__sparc64__)
@@ -336,7 +366,7 @@ extern void __TR_clear_cache();
 #define TRAMP_LENGTH 32
 #define TRAMP_ALIGN 4
 #endif
-#ifdef __rs6000sysv4__
+#if defined(__rs6000sysv4__) || defined(__powerpcnetbsd__)
 #define TRAMP_LENGTH 24
 #define TRAMP_ALIGN 4
 #endif
@@ -921,6 +951,39 @@ __TR_function alloc_trampoline_r (address, data0, data1)
 #define tramp_data(function)  \
   hilo(*(unsigned short *) (function + 2), *(unsigned short *) (function + 6))
 #endif
+#ifdef __powerpcnetbsd__
+  /* function:
+   *    {liu|lis} 13,hi16(<data>)		3D A0 hi16(<data>)
+   *    {oril|ori} 13,13,lo16(<data>)		61 AD lo16(<data>)
+   *    {liu|lis} 0,hi16(<address>)		3C 00 hi16(<address>)
+   *    {oril|ori} 0,0,lo16(<address>)		60 00 lo16(<address>)
+   *    mtctr 0					7C 09 03 A6
+   *    bctr					4E 80 04 20
+   */
+  *(short *) (function + 0) = 0x3DA0;
+  *(short *) (function + 2) = (unsigned long) data >> 16;
+  *(short *) (function + 4) = 0x61AD;
+  *(short *) (function + 6) = (unsigned long) data & 0xffff;
+  *(short *) (function + 8) = 0x3C00;
+  *(short *) (function +10) = (unsigned long) address >> 16;
+  *(short *) (function +12) = 0x6000;
+  *(short *) (function +14) = (unsigned long) address & 0xffff;
+  *(long *)  (function +16) = 0x7C0903A6;
+  *(long *)  (function +20) = 0x4E800420;
+#define is_tramp(function)  \
+  *(unsigned short *) (function + 0) == 0x3DA0 && \
+  *(unsigned short *) (function + 4) == 0x61AD && \
+  *(unsigned short *) (function + 8) == 0x3C00 && \
+  *(unsigned short *) (function +12) == 0x6000 && \
+  *(unsigned long *)  (function +16) == 0x7C0903A6 && \
+  *(unsigned long *)  (function +20) == 0x4E800420
+#define hilo(hiword,loword)  \
+  (((unsigned long) (hiword) << 16) | (unsigned long) (loword))
+#define tramp_address(function)  \
+  hilo(*(unsigned short *) (function +10), *(unsigned short *) (function +14))
+#define tramp_data(function)  \
+  hilo(*(unsigned short *) (function + 2), *(unsigned short *) (function + 6))
+#endif
 #ifdef __rs6000aix__
   /* function:
    *    .long .tramp_r
