@@ -16,19 +16,35 @@
 ; along with this program; if not, write to the Free Software Foundation,
 ; Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 ;
-; $Id: oracle.lisp,v 1.1 2002/09/11 14:27:53 hin Exp $
+; $Id: oracle.lisp,v 1.13 2003/08/27 13:55:12 hin Exp $
 
 (defpackage "ORACLE"
   (:documentation
    "CLISP Oracle interface from Alma Mater Software, Inc.  Inquiries to: John Hinsdale <hin@alma.com>")
   (:use "LISP" "FFI")
   (:export "CONNECT" "DISCONNECT" "RUN-SQL"
-           "DO-ROWS" "FETCH" "PEEK" "COLUMNS" "EOF"
+           "DO-ROWS" "FETCH" "FETCH-ALL" "PEEK" "COLUMNS" "EOF"
            "INSERT-ROW" "UPDATE-ROW" "ROW-COUNT"
-           "WITH-TRANSACTION" "COMMIT" "ROLLBACK" "AUTO-COMMIT"))
+           "WITH-TRANSACTION" "COMMIT" "ROLLBACK" "AUTO-COMMIT"
+	   "SQLCOL-NAME" "SQLCOL-TYPE" "SQLCOL-SIZE" "SQLCOL-SCALE" "SQLCOL-PRECISION" "SQLCOL-NULL_OK" ))
 
 (in-package "ORACLE")
 
+; Use "C" as foreign language
+(default-foreign-language :stdc)
+
+; Inline everything for speed
+(proclaim '(inline
+aref-null array-to-hash auto-commit auto-commit-nocheck cat
+check-connection check-pairs check-success check-unique-elements
+column-names columns comma-list-of-keys commit commit-nocheck connect
+connection-key convert-type c-truth curconn disconnect do-rows-col
+do-rows-index-of do-rows-var eof fetch flatten from-sqlval
+gethash-required hash-combine hash-to-sqlparam-array if-null
+insert-row is-select-query join lisp-truth nl out out-nl pairs-to-hash
+peek rollback rollback-nocheck row-count row-to-result rowval run-sql
+to-sqlval to-string update-row valid-symbol
+))
 
 ;; GLOBALS
 
@@ -45,6 +61,7 @@
   connection     ; "C" library handle for Oracle operations
   fetch-called   ; Flag: has fetch been called yet
   pending-row    ; Look-ahead row (for PEEK)
+  colinfo        ; Cache the column info for speed
   hkey           ; Key in global hash (useful for removal)
 )
 
@@ -80,8 +97,8 @@
 
 ; CONNECT
 
-(defun connect (user password server &optional schema (auto-commit t) (prefetch-buffer-bytes 0))
-"(ORACLE::CONNECT user password server &optional schema auto-commit prefetch-buffer-bytes)
+(defun connect (user password server &optional schema (auto-commit t) (prefetch-buffer-bytes 0) (long-len -1) (truncate-ok t))
+"(ORACLE::CONNECT user password server &optional schema auto-commit prefetch-buffer-bytes long-len truncate-ok)
 
 Connect to an Oracle database.  All subsequent operations will affect
 this database.  A single program can access several different
@@ -101,16 +118,28 @@ Optional arguments:
                  the database as if you were some other user.
     auto-commit  Commit after every operation (default: T).  Set this to NIL if
                  you intend to do transactions, and call call COMMIT explicitly.
-    prefetch-    Number of bytes to cache from SELECT fetches (default: 1,000,000)
+    prefetch-    Number of bytes to cache from SELECT fetches (default: 64 Kbytes)
     buffer-      If you are very short of memory, or have a slow connection to Oracle,
-    bytes        reduce this to 10000 or so.
+    bytes        you can reduce this to 10,000 or so.  Alternatively, if you have a
+                 fast connection to Oracle and regularly do large queries, you can
+                 increase throughput by increasing this value.
+    long-len     Number of bytes to fetch for \"long\" (LONG, [BC]LOB) types.
+                 Long data that exceeds this size will raise an error,
+                 or be truncated depending on the value of truncate-ok
+                 (below).  Setting this parameter to zero and disallowing truncation
+                 will disable long fetching entirely.  If NIL or negative, defaults to
+                 500k bytes.
+    truncate-ok  If set, allow truncation of LONG columns to long-len (default: NIL).
 
-Returns: T if new connection was created, NIL if cache was used.
+Returns: T if a cached connection was re-used (NIL if a new connection
+         was created and cached).
 "
-  (when *oracle-in-transaction* (error "CONNECT not allowed inside WITH-TRANSACTION"))
+  (when *oracle-in-transaction* (db-error "CONNECT not allowed inside WITH-TRANSACTION"))
 
   ; Default current schema
-  (if (null schema) (setq schema user))
+  (if (null schema) (setf schema user))
+  (if (null long-len) (setf long-len -1))
+  (if (null prefetch-buffer-bytes) (setf prefetch-buffer-bytes 0))
 
   ; Set up global connection cache
   (if (null *oracle-connection-cache*)
@@ -122,12 +151,12 @@ Returns: T if new connection was created, NIL if cache was used.
          (result t))
     (when (null conn)
           ; Connect to database
-          (let ((handle (oracle_connect user schema password server prefetch-buffer-bytes (c-truth auto-commit))))
+          (let ((handle (oracle_connect user schema password server prefetch-buffer-bytes (c-truth auto-commit) long-len (c-truth truncate-ok))))
             (when (not (lisp-truth (oracle_success handle)))
                    ; Retry connection
                    ; ... TODO: implement retry logic here
                    ; Failed all attempts; give up
-                  (error (oracle_last_error handle)))
+                  (db-error (oracle_last_error handle)))
             ; OK: cache the connection
             (setf conn (make-db :connection handle :hkey hkey))
             (setf (gethash hkey *oracle-connection-cache*) conn)
@@ -150,8 +179,8 @@ DISCONNECT may not be called inside the WITH-TRANSACTION macro.
 Arguments: none.
 Returns: NIL
 "
-  (when *oracle-in-transaction* (error "DISCONNECT not allowed inside WITH-TRANSACTION"))
-  (when (not (null (curconn)))
+  (when *oracle-in-transaction* (db-error "DISCONNECT not allowed inside WITH-TRANSACTION"))
+  (when (curconn)
         (oracle_disconnect (curconn))
         (check-success)
         ; Remove connection from the hash table
@@ -201,6 +230,7 @@ for SELECT statements.
                    (c-truth (not is-select)))
   (setf (db-fetch-called *oracle-connection*) nil)
   (setf (db-pending-row *oracle-connection*) nil)
+  (setf (db-colinfo *oracle-connection*) nil)
   (check-success)
 
   ; Get the row count for the result
@@ -223,22 +253,49 @@ switch the conneciton to a different database other than the one that
 was used to do the SELECT.  This is useful for reading from one
 database while writing to another.
 
+When specifying variables to which to bind column values, instead of a
+single symbol, a pair (bound-var \"column-name\") can be specified which
+will cause values from the given column name to be bound to the given
+variable.  This is for unusual cases where a Lisp symbol cannot be
+created with the same for the column (e.g., a column names "T") and
+when it is inconvenient of impossible to alias the column with
+\"SELECT ... AS\"
+
 "
+  ; COMPILE TIME CHECKS
+  ; Validate both variable list and column aliases are unique (at
+  ; compile time)
+  (check-unique-elements (map 'list #'do-rows-var vars))
+
+  ; Conceivably the caller MIGHT want to bind two different loop
+  ; variables to the same SELECTed column, but more likely that is a bug
+  ; on his part, so don't allow it.
+  (check-unique-elements (map 'list #'do-rows-col vars))
+
+  ; Declare variables and bind to fetch from appropriate array index.
+  ; Generate a map of the bound vars to gensyms which contain the index of that
+  ; var into the fetched array.
   (let ((fetch-result (gensym))
-        (colhash (gensym))
-        (saved-oracle-connection (gensym)))
+        (saved-oracle-connection (gensym))
+        (index-vars (make-hash-table)))
+    (dolist (v vars)
+            (setf (gethash (to-string (do-rows-var v)) index-vars) (gensym)))
     (list 'let
-          `((,saved-oracle-connection *oracle-connection*))
+          ; Declare saved Oracle connection and calculated array indices OUTSIDE fetch loop
+          (append `((,saved-oracle-connection *oracle-connection*))
+                        (map 'list
+                             #'(lambda (v) (list (gethash (to-string (do-rows-var v)) index-vars)
+                                                 (list 'do-rows-index-of (list 'quote v))))
+                             vars))
+          ; Emit the DO loop itself
           (append (list 
                    'do*
-                   (append (list `(,fetch-result (fetch 'hash) (fetch 'hash))
-                                 `(,colhash (null-hash ,fetch-result) (null-hash ,fetch-result)))
-                           (map 'list #'(lambda (col)
-                                          (let ((iter (list 'gethash-required
-                                                            (list 'quote col)
-                                                            colhash)))
-                                            (list col iter iter)))
-                                vars))
+                   (append (list `(,fetch-result (fetch 'array) (fetch 'array)))
+                             (map 'list 
+                                  #'(lambda (k)
+                                      (let ((iter (list 'aref-null fetch-result (gethash (to-string (do-rows-var k)) index-vars))))
+                                        (list (do-rows-var k) iter iter)))
+                                  vars))
                    (list `(null ,fetch-result) '(row-count)))
                   (if (atom body) (list body) body)
                   (list `(setf *oracle-connection* ,saved-oracle-connection))))))
@@ -273,6 +330,7 @@ Oracle data types are converted to Lisp datatypes as follows:
     - Strings (char, varchar, varchar2) are left as Lisp strings
     - Dates are converted to strings of the form \"YYYY-MM-DD HH:MM:SS\"
       where HH is 24-hour form.
+    - RAW and LONG RAW are converted to hexadecimal strings
 
 Returns NIL if no rows are left (the EOF predicate can be before
 a fetch to test this condition).
@@ -300,7 +358,7 @@ Arguments: none
      ; Do a real fetch from Oracle
      (t (let ((fetch_status (oracle_fetch_row (curconn))))
           (cond ((not (lisp-truth fetch_status))
-                 (error (oracle_last_error (curconn))))
+                 (db-error (oracle_last_error (curconn))))
                 ((= fetch_status 2)
                  ; Newly arrived at EOF - do nothing
                  )
@@ -325,6 +383,34 @@ Arguments: none
 
 ;----------------------------------------------
 
+; FETCH-ALL
+(defun fetch-all (&optional max-rows (result-type 'ARRAY) (item-type 'ARRAY))
+"(ORACLE:FETCH-ALL(&optional max-rows (result-type 'ARRAY) (item-type 'ARRAY))
+
+Fetch all rows from a query and return result as a sequence of
+sequences.  Arguments (all optional) are:
+
+   max-rows      Maximum number of rows to fetch
+   result-type   Sequence type of row set 'ARRAY (default) or 'LIST
+   item-type     Sequence type of columns per row,'ARRAY (default) or 'LIST
+"
+  (check-connection "fetch all rows of data")
+  (when (not (find result-type '(array list)))
+	(db-error (cat "Result type '" result-type "' should be 'ARRAY or 'LIST")))
+  (when (not (find item-type '(array list)))
+	(db-error (cat "Item type '" item-type "' should be 'ARRAY or 'LIST")))
+
+  (do ((result (make-array 100 :element-type item-type
+			   :fill-pointer 0 :adjustable t))
+       (count 0 (1+ count))
+       (row (oracle:fetch) (oracle:fetch)))
+      ((or (null row) (when max-rows (>= count max-rows)))
+       (coerce result result-type))
+      (vector-push-extend (coerce row item-type)
+				      result)))
+
+;----------------------------------------------
+
 ; PEEK
 (defun peek (&optional (result-type 'ARRAY))
 "(ORACLE:PEEK (&optional (result-type 'ARRAY)))
@@ -367,15 +453,19 @@ slots:
 Arguments: none
 "
   (check-connection "get column information")
-  (let ((result (oracle_column_info (curconn))))
-    (check-success)
-    ; Convert C truth to Lisp for export
-    (map-into result
-              #'(lambda (col)
-                  (setf (sqlcol-null_ok col) (lisp-truth (sqlcol-null_ok col)))
-                  col)
-              result)
-    result))
+  (let ((cached-info (db-colinfo *oracle-connection*)))
+    (if cached-info
+        cached-info
+      (let ((result (oracle_column_info (curconn))))
+        (check-success)
+        ; Convert C truth to Lisp for export
+        (map-into result
+                  #'(lambda (col)
+                      (setf (sqlcol-null_ok col) (lisp-truth (sqlcol-null_ok col)))
+                      col)
+                  result)
+        (setf (db-colinfo *oracle-connection*) result)
+        result))))
 
 ;----------------------------------------------
 
@@ -414,9 +504,9 @@ given the default Oracle value, or NULL.
 
 Returns: the number of rows inserted (i.e., 1).
 "
-  (when (null vals) (error "NULL name -> value map given"))
+  (when (null vals) (db-error "NULL name -> value map given"))
   (setf vals (check-pairs vals))
-  (when (= 0 (hash-table-count vals)) (error "Empty column map given"))
+  (when (= 0 (hash-table-count vals)) (db-error "Empty column map given"))
   ; Build the INSERT statement
   (let ((sql (cat "INSERT INTO " table " ("
                   (comma-list-of-keys vals)
@@ -443,9 +533,9 @@ a match on a primary key, e.g.: \"pk_column = :pk_val\".
 Returns: the number of rows updated.
 
 "
-  (when (null vals) (error "NULL name -> value map given"))
+  (when (null vals) (db-error "NULL name -> value map given"))
   (setf vals (check-pairs vals))
-  (when (= 0 (hash-table-count vals)) (error "Empty column map given"))
+  (when (= 0 (hash-table-count vals)) (db-error "Empty column map given"))
 
   ; Build the UPDATE statement
   (let ((sql (cat "UPDATE " table " SET "))
@@ -476,7 +566,7 @@ Arguments: none
   (let ((rowcount (oracle_rows_affected (curconn))))
     (check-success)
     ; Maybe adjust downward to account for lookahead row
-    (if (not (null (db-pending-row *oracle-connection*)))
+    (if (db-pending-row *oracle-connection*)
         (- rowcount 1)
       rowcount)))
 
@@ -497,7 +587,7 @@ WITH-TRANSACTION macro.
 
 Arguments: (Boolean) Whether to enable auto-commit.
 "
-  (when *oracle-in-transaction* (error "Setting of AUTO-COMMIT not allowed inside WITH-TRANSACTION"))
+  (when *oracle-in-transaction* (db-error "Setting of AUTO-COMMIT not allowed inside WITH-TRANSACTION"))
   (auto-commit-nocheck enable))
 
 ; Private version that does not check if in transaction
@@ -529,7 +619,7 @@ body.
         (result (gensym)))
     `(progn
        ; Check nesting
-       (when *oracle-in-transaction* (error "Nesting of WITH-TRANSACTION is not allowed."))
+       (when *oracle-in-transaction* (db-error "Nesting of WITH-TRANSACTION is not allowed."))
        (let ((,prev-auto-commit t)
              (,commit-ok nil))
          (unwind-protect
@@ -562,7 +652,7 @@ called inside the WITH-TRANSACTION macro. Always returns NIL.
 
 Argument: none
 "
-  (when *oracle-in-transaction* (error "COMMIT not allowed inside WITH-TRANSACTION"))
+  (when *oracle-in-transaction* (db-error "COMMIT not allowed inside WITH-TRANSACTION"))
   (commit-nocheck))
 
 (defun commit-nocheck ()
@@ -583,7 +673,7 @@ called insde the WITH-TRANSACTION macro.  Always returns NIL.
 
 Argument: none
 "
-  (when *oracle-in-transaction* (error "ROLLBACK not allowed inside WITH-TRANSACTION"))
+  (when *oracle-in-transaction* (db-error "ROLLBACK not allowed inside WITH-TRANSACTION"))
   (rollback-nocheck))
 
 (defun rollback-nocheck ()
@@ -595,6 +685,19 @@ Argument: none
 
 ; =-=-=-=-=-=-=-   INTERNAL FUNCTIONS BELOW     =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 
+; Helper functions for DO-ROWS
+(defun do-rows-var (v) (if (atom v) v (car v)))
+(defun do-rows-col (v) (if (atom v) v (cadr v)))
+(defun do-rows-index-of (v)
+  (let ((i (position (to-string (do-rows-col v))
+                     (map 'array
+                          #'(lambda (x) (to-string (sqlcol-name x)))
+                          (columns))
+                     :test #'equal)))
+    (when (null i)
+          (db-error (cat "DO-ROWS: Column '" (do-rows-col v)
+                      "' does not occur in query.  Allowed columns are:~%" (column-names))))
+    i))
 ; COLUMN-NAMES
 ; Get list of column names, one per line.
 (defun column-names ()
@@ -608,7 +711,7 @@ Argument: none
       nil
     (multiple-value-bind
      (val exists) (gethash (to-string key) hash)
-     (when (not exists) (error (cat "DO-ROWS: bound variable '" key "' does not occur in the query."
+     (when (not exists) (db-error (cat "DO-ROWS: bound variable '" key "' does not occur in the query."
                                     "~%The allowed column/variable names are:~%~%" (column-names)
                                     "~%")))
      val)))
@@ -618,7 +721,8 @@ Argument: none
 (defun row-to-result (row result-type)
   (cond ((null row) nil)
         ((eq result-type 'ARRAY) row)
-        ((eq result-type 'HASH) (pairs-to-hash (row-to-result row 'PAIRS)))
+        ; ((eq result-type 'HASH) (pairs-to-hash (row-to-result row 'PAIRS)))
+        ((eq result-type 'HASH) (array-to-hash (row-to-result row 'ARRAY)))
         ((eq result-type 'PAIRS)
          (let ((colinfo (oracle_column_info (curconn))))
            (check-success)
@@ -628,14 +732,14 @@ Argument: none
                     #'(lambda (col rowval)
                         (list (sqlcol-name col) rowval))
                     colinfo row)))))
-        (t (error (cat "Invalid result type '" result-type "' given - should be 'ARRAY, 'PAIRS or 'HASH")))))
+        (t (db-error (cat "Invalid result type '" result-type "' given - should be 'ARRAY, 'PAIRS or 'HASH")))))
 
 
 ; CHECK-SUCCESS
 ; Check Oracle success code after calling a function.  Assumes (check-connection) was called!
 (defun check-success ()
   (if (not (lisp-truth (oracle_success (curconn))))
-      (error (oracle_last_error (curconn))))
+      (db-error (oracle_last_error (curconn))))
   t)
 
 ; Convert Oracle type based on sqlcol data type.  Oracle numerics are converted
@@ -646,9 +750,9 @@ Argument: none
     (cond ((null val) nil)
           ((find dtype '("NUMBER" "INTEGER" "FLOAT") :test #'equal)
            (read-from-string val))
-          ((find dtype '("VARCHAR" "DATE" "CHAR" "VARCHAR2") :test #'equal)
+          ((find dtype '("VARCHAR" "DATE" "CHAR" "VARCHAR2" "LONG" "RAW" "LONG RAW" "BLOB" "CLOB" "BFILE") :test #'equal)
            val)
-          (t (error (cat "Unsupported data type '" dtype "'"))))))
+          (t (db-error (cat "Unsupported data type '" dtype "'"))))))
 
 ; TO-SQLVAL
 ; Return a SQL val for LISP object, handling null case
@@ -681,9 +785,9 @@ Argument: none
     (loop for key being the hash-keys of h do
           (let ((val (gethash key h)))
             (when (not (atom key))
-                  (error "Non-atom parameter name in bind-parameter hash"))
+                  (db-error "Non-atom parameter name in bind-parameter hash"))
             (when (not (atom val))
-                  (error "Non-atom parameter value in bind-parameter hash"))
+                  (db-error "Non-atom parameter value in bind-parameter hash"))
             (setf (aref result i) (make-sqlparam :name (to-string key) :value (to-sqlval val)))
             (incf i)))
     result))
@@ -692,7 +796,7 @@ Argument: none
 ; Check we are connected before doing an operation that requires a connection
 (defun check-connection (&optional action)
   (if (null (curconn))
-      (error (cat "Attempt to "
+      (db-error (cat "Attempt to "
                   (if-null action "perform database operation")
                   " when not connected to any database"))))
 
@@ -712,12 +816,12 @@ Argument: none
       (loop for p in plist do
             (let ((key (string-upcase (to-string (first p))))
                   (value (second p)))
-              (when (not (valid-symbol key)) (error (cat "Column or parameter '" key "' is not a valid Lisp symbol name."
+              (when (not (valid-symbol key)) (db-error (cat "Column or parameter '" key "' is not a valid Lisp symbol name."
                                                          "~%Consider using SELECT ... " key " AS <column alias>")))
               ; Check uniqueness
               (multiple-value-bind 
                (curval already-there) (gethash key result)
-               (when already-there (error (cat "Column or parameter '" key
+               (when already-there (db-error (cat "Column or parameter '" key
                                                "' appears twice in list of (name, value) pairs,~%first with value '"
                                                curval "' and again with value '" value "'.  Columns/parameters given were:~%"
                                                (join "~%" (map 'list #'car plist))
@@ -729,10 +833,10 @@ Argument: none
 ; CHECK-PAIRS
 ; Convert pairs to hash if needed
 (defun check-pairs (p)
-  (cond ((null p) (make-hash-table))
+  (cond ((null p) (make-hash-table :test #'equal))
         ((eq (type-of p) 'HASH-TABLE) p)
         ((eq (type-of p) 'CONS) (pairs-to-hash p))
-        (t (error (cat "Invalid type for name -> value map: '" (type-of p) "' - should be hash or list of pairs.")))))
+        (t (db-error (cat "Invalid type for name -> value map: '" (type-of p) "' - should be hash or list of pairs.")))))
 
 ; COMMA-LIST-OF-KEYS
 ; Return keys of hash table as comma-separated list.  If flag given,
@@ -757,68 +861,73 @@ Argument: none
 
 ; =-=-=-=-=-=-=-   C WRAPPER FUNCTIONS  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 
+(c-lines "#include <oci.h>~%")
+(c-lines "#include \"oiface.h\"~%")
+
 ; CONNECT
-(def-c-call-out oracle_connect (:arguments (user           c-string)
-                                           (schema         c-string)
-                                           (password       c-string)
-                                           (server         c-string)
-                                           (prefetch_bytes int)
-                                           (auto_commit    int))
-                               (:return-type c-pointer))
+(def-call-out oracle_connect (:arguments (user           c-string)
+                                         (schema         c-string)
+                                         (password       c-string)
+                                         (server         c-string)
+                                         (prefetch_bytes int)
+                                         (auto_commit    int)
+                                         (long_len       int)
+                                         (truncate_ok    int))
+                             (:return-type c-pointer))
 
 ; DISCONNECT
-(def-c-call-out oracle_disconnect (:arguments (db c-pointer))
-                                  (:return-type int  ))
+(def-call-out oracle_disconnect (:arguments (db c-pointer))
+                                (:return-type int  ))
 
 ; RUN SQL
-(def-c-call-out oracle_exec_sql (:arguments (db         c-pointer)
-                                            (sql        c-string)
-                                            (params     (c-array-ptr (c-ptr sqlparam)))
-                                            (is_command int))
-                                (:return-type int))
-
-; NO. OF COLUMNS
-(def-c-call-out oracle_ncol (:arguments (db c-pointer))
-                            (:return-type int))
-
-; COLUMN INFO
-(def-c-call-out oracle_column_info (:arguments (db c-pointer))
-                                   (:return-type (c-array-ptr (c-ptr sqlcol))))
-
-; FETCH
-(def-c-call-out oracle_fetch_row (:arguments (db c-pointer))
-                                 (:return-type int))
-
-; EOF
-(def-c-call-out oracle_eof (:arguments (db c-pointer))
-                           (:return-type int))
-
-; SUCCESS
-(def-c-call-out oracle_success (:arguments (db c-pointer))
-                               (:return-type int))
-
-; ROW VALUES
-(def-c-call-out oracle_row_values (:arguments (db c-pointer))
-                                  (:return-type (c-array-ptr (c-ptr sqlval))))
-
-; NO. ROWS AFFECTED
-(def-c-call-out oracle_rows_affected (:arguments (db c-pointer))
-                                     (:return-type int))
-
-; COMMIT
-(def-c-call-out oracle_commit (:arguments (db c-pointer))
+(def-call-out oracle_exec_sql (:arguments (db         c-pointer)
+                                          (sql        c-string)
+                                          (params     (c-array-ptr (c-ptr sqlparam)))
+                                          (is_command int))
                               (:return-type int))
 
-(def-c-call-out oracle_rollback (:arguments (db c-pointer))
-                                (:return-type int))
+; NO. OF COLUMNS
+(def-call-out oracle_ncol (:arguments (db c-pointer))
+                          (:return-type int))
 
-(def-c-call-out oracle_set_auto_commit (:arguments (db c-pointer)
-                                                   (auto_commit int))
-                                       (:return-type int))
+; COLUMN INFO
+(def-call-out oracle_column_info (:arguments (db c-pointer))
+                                 (:return-type (c-array-ptr (c-ptr sqlcol))))
+
+; FETCH
+(def-call-out oracle_fetch_row (:arguments (db c-pointer))
+                               (:return-type int))
+
+; EOF
+(def-call-out oracle_eof (:arguments (db c-pointer))
+                         (:return-type int))
+
+; SUCCESS
+(def-call-out oracle_success (:arguments (db c-pointer))
+                             (:return-type int))
+
+; ROW VALUES
+(def-call-out oracle_row_values (:arguments (db c-pointer))
+                                (:return-type (c-array-ptr (c-ptr sqlval))))
+
+; NO. ROWS AFFECTED
+(def-call-out oracle_rows_affected (:arguments (db c-pointer))
+                                   (:return-type int))
+
+; COMMIT
+(def-call-out oracle_commit (:arguments (db c-pointer))
+                            (:return-type int))
+
+(def-call-out oracle_rollback (:arguments (db c-pointer))
+                              (:return-type int))
+
+(def-call-out oracle_set_auto_commit (:arguments (db c-pointer)
+                                                 (auto_commit int))
+                                     (:return-type int))
 
 ; ERROR
-(def-c-call-out oracle_last_error (:arguments (db c-pointer))
-                                  (:return-type c-string))
+(def-call-out oracle_last_error (:arguments (db c-pointer))
+                                (:return-type c-string))
 
 
 ; =-=-=-=-=-=-=-   LOW LEVEL UTILITY FUNCTIONS  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
@@ -827,10 +936,9 @@ Argument: none
 ; Default a null value.  Is there a better Lisp built-in for this?
 (defun if-null (value default) (if (null value) default value))
 
-; NULL-HASH
-; Map NIL to empty hash table
-(defun null-hash (h)
-  (if-null h (make-hash-table)))
+; AREF-NULL
+; Do an AREF, but allow array to be null, in which case return NIL
+(defun aref-null (a i) (if (null a) nil (aref a i)))
 
 ; HASH-COMBINE
 ; Combine two hash table.  Keys of the second hash will overwrite.
@@ -851,13 +959,41 @@ Argument: none
 ; TO-STRING
 ; Convert object to a string; NIL -> ""
 (defun to-string (s)
-  (if (null s) "" (format nil "~A" s)))
+  (cond ((null s) "")
+        ((stringp s) s)
+        ((symbolp s) (symbol-name s))
+        (t (format nil "~A" s))))
 
 ; CAT
 ; Concatenate strings
 (defun cat (&rest args) 
   (apply #'concatenate 'string (mapcar #'to-string (flatten args))))
 
+; ARRAY-TO-HASH
+; Convert array of row values to hash using column info
+(defun array-to-hash (row)
+  (if (null row)
+      nil
+    (let* ((cols (columns))
+           (n (length row))
+           (result (make-hash-table :test #'equal :size n)))
+      (loop for i from 0 to (- n 1) do
+            (setf (gethash (to-string (sqlcol-name (aref cols i))) result) (aref row i)))
+      result)))
+
+; CHECK-UNIQUE-ELEMENTS
+; Does list consist of unqiue, non-null elements
+(defun check-unique-elements (l)
+  (let ((h (make-hash-table :test #'equal)))
+    (dolist (elt l)
+            (when (null elt)
+                  (db-error "Null element in column/variable list"))
+            (when (gethash (to-string elt) h)
+                  (db-error (cat "DO-ROWS: Parameter/column '" elt "' occurs more than once in bound columns/variables:~%"
+                              (join "~%" l))))
+            (setf (gethash (to-string elt) h) t))
+    t))
+
 ; JOIN
 ; Join a sequence of strings into one, separating with delimeter
 ; I'll probably get shot for this implementation.  Better way?
@@ -909,4 +1045,8 @@ Argument: none
 ; Return newline
 (defun nl () (format nil "~%"))
 
+; DB-ERROR - Throw an error
+(defun db-error (message)
+  (error message))
+
 ; End of oracle.lisp
