@@ -8,13 +8,15 @@
 
 (default-foreign-language :stdc)
 
+(c-lines "#include <fnmatch.h>~%")
+
 (def-call-out fnmatch
     (:arguments (pattern c-string)
                 (string c-string)
                 (flags int))
   (:return-type int))
 
-; flags values
+;; flags values
 (defconstant FNM_PATHNAME     1)
 (defconstant FNM_FILE_NAME    1)
 (defconstant FNM_NOESCAPE     2)
@@ -23,16 +25,13 @@
 (defconstant FNM_CASEFOLD    16)
 
 (defun match (pattern string &key (start 0) (end nil) (case-insensitive nil))
-  ; Prepare the string.
+  ;; Prepare the string.
   (unless (and (eql start 0) (null end))
     (unless end (setq end (length string)))
     (setq string (make-array (- end start) :element-type 'character
                                            :displaced-to string
-                                           :displaced-index-offset start
-  ) )            )
-  ; Match.
+                                           :displaced-index-offset start)))
+  ;; Match.
   (zerop
     (fnmatch pattern string
-             (logior FNM_PATHNAME (if case-insensitive FNM_CASEFOLD 0))
-  ) )
-)
+             (logior FNM_PATHNAME (if case-insensitive FNM_CASEFOLD 0)))))
