@@ -1,4 +1,4 @@
-;;;; Einige Definitionen von Standard-Funktionen in LISP
+;;;; some definitions for standard functions in LISP
 ;;;; 1.8.1989, 2.9.1989, 8.10.1989
 
 (in-package "EXT")
@@ -11,59 +11,50 @@
 
 (in-package "SYSTEM")
 
-;;; Funktionen für Symbole (Kapitel 10)
+;;; functions for symbols (Chapter 10)
 
-(defun copy-symbol (symbol &optional flag)
-                   ;; Common LISP, S. 169
+(defun copy-symbol (symbol &optional flag) ;; Common LISP, p. 169
   (let ((sym (make-symbol (symbol-name symbol))))
     (when flag
       (when (boundp symbol) (sys::set-symbol-value sym (%symbol-value symbol)))
       (when (fboundp symbol) (sys::%putd sym (symbol-function symbol)))
-      (sys::%putplist sym (copy-list (symbol-plist symbol)))
-    )
-    sym
-) )
+      (sys::%putplist sym (copy-list (symbol-plist symbol))))
+    sym))
 
-(let ((gentemp-count 0))
+(let ((gentemp-count 0)) ;; Common LISP, p. 170
   (defun gentemp (&optional (prefix "T") (package *package*))
-                 ;; Common LISP, S. 170
     (loop
       (setq gentemp-count (1+ gentemp-count))
       (multiple-value-bind (sym flag)
         (intern
           (string-concat prefix
-            (write-to-string gentemp-count :base 10 :radix nil :readably nil)
-          )
-          package
-        )
-        (unless flag (return sym))
-) ) ) )
+            (write-to-string gentemp-count :base 10 :radix nil :readably nil))
+          package)
+        (unless flag (return sym))))))
 
 
-;;; Macros für Packages (Kapitel 11), S. 187-188
-
+;;; macros for packages (Chapter 11), p. 187-188
 (defmacro do-symbols ((var &optional (packageform '*package*) (resultform nil))
                       &body body &environment env)
-  (multiple-value-bind (body-rest declarations) (system::parse-body body nil env)
+  (multiple-value-bind (body-rest declarations)
+      (system::parse-body body nil env)
     (let ((packvar (gensym)))
       `(BLOCK NIL
          (LET ((,packvar ,packageform))
            (LET ((,var NIL))
              (DECLARE (IGNORABLE ,var) ,@declarations)
              (SYSTEM::MAP-SYMBOLS
-               #'(LAMBDA (,var)
-                   ,@(if declarations `((DECLARE ,@declarations)) '())
-                   ,@body-rest
-                 )
-               ,packvar
-             )
-             ,resultform
-       ) ) )
-) ) )
+              #'(LAMBDA (,var)
+                  ,@(if declarations `((DECLARE ,@declarations)) '())
+                  (TAGBODY ,@body-rest))
+               ,packvar)
+             ,resultform))))))
 
-(defmacro do-external-symbols ((var &optional (packageform '*package*) (resultform nil))
+(defmacro do-external-symbols ((var &optional (packageform '*package*)
+                                    (resultform nil))
                                &body body &environment env)
-  (multiple-value-bind (body-rest declarations) (system::parse-body body nil env)
+  (multiple-value-bind (body-rest declarations)
+      (system::parse-body body nil env)
     (let ((packvar (gensym)))
       `(BLOCK NIL
          (LET ((,packvar ,packageform))
@@ -72,56 +63,46 @@
              (SYSTEM::MAP-EXTERNAL-SYMBOLS
                #'(LAMBDA (,var)
                    ,@(if declarations `((DECLARE ,@declarations)) '())
-                   ,@body-rest
-                 )
-               ,packvar
-             )
-             ,resultform
-       ) ) )
-) ) )
+                   (TAGBODY ,@body-rest))
+               ,packvar)
+             ,resultform))))))
 
 (defmacro do-all-symbols ((var &optional (resultform nil))
                           &body body &environment env)
-  (multiple-value-bind (body-rest declarations) (system::parse-body body nil env)
+  (multiple-value-bind (body-rest declarations)
+      (system::parse-body body nil env)
     `(BLOCK NIL
        (LET ((,var NIL))
          (DECLARE (IGNORABLE ,var) ,@declarations)
          (SYSTEM::MAP-ALL-SYMBOLS
            #'(LAMBDA (,var)
                ,@(if declarations `((DECLARE ,@declarations)) '())
-               ,@body-rest
-             )
-         )
-         ,resultform
-     ) )
-) )
+               (TAGBODY ,@body-rest)))
+         ,resultform))))
 
 ;;; <HS>/Body/mac_with-package-iterator.html
 (defmacro with-package-iterator ((name pack-list &rest types) &body body)
   (unless types
     (error-of-type 'source-program-error
       (TEXT "missing symbol types (~S/~S/~S) in ~S")
-      ':internal ':external ':inherited 'with-package-iterator
-  ) )
+      ':internal ':external ':inherited 'with-package-iterator))
   (dolist (symboltype types)
     (case symboltype
       ((:INTERNAL :EXTERNAL :INHERITED))
       (t (error-of-type 'source-program-error
            (TEXT "~S: flag must be one of the symbols ~S, ~S, ~S, not ~S")
-           'with-package-iterator ':internal ':external ':inherited symboltype
-  ) ) )  )
+           'with-package-iterator ':internal ':external ':inherited
+           symboltype))))
   (let ((iterfun (gensym "WPI")))
-    `(let ((,iterfun (package-iterator-function ,pack-list ',(remove-duplicates types))))
+    `(let ((,iterfun (package-iterator-function
+                      ,pack-list ',(remove-duplicates types))))
        (macrolet ((,name () '(funcall ,iterfun)))
-         ,@body
-     ) )
-) )
+         ,@body))))
 (defun package-iterator-function (pack-list symbol-types)
   (let ((iterstates
           (mapcar #'(lambda (pack) (sys::package-iterator pack symbol-types))
-                  (if (listp pack-list) pack-list (list pack-list))
-       )) )
-    ; The iterstates list is cdr'ed down during the iteration.
+                  (if (listp pack-list) pack-list (list pack-list)))))
+    ;; The iterstates list is cdr'ed down during the iteration.
     #'(lambda ()
         (loop
           (if iterstates
@@ -129,18 +110,15 @@
                 (sys::package-iterate (car iterstates))
               (if more
                 (return (values more symb acc (svref (car iterstates) 4)))
-                (pop iterstates)
-            ) )
-            (return nil)
-      ) ) )
-) )
+                (pop iterstates)))
+            (return nil))))))
 
 ;; The list of packages that will be locked by SAVEINITMEM.
 ;; Also the default packages to unlock by WITHOUT-PACKAGE-LOCK.
 (defvar *system-package-list*
   '("SYSTEM" "LISP" "EXT" "CUSTOM" "I18N" "GRAY" "CHARSET" "CLOS"
     #+sockets "SOCKET" #+generic-streams "GSTREAM" #+syscalls "POSIX"
-    #+ffi "FFI" #+amiga "AFFI" #+dir-key "LDAP" #+screen "SCREEN"))
+    #+ffi "FFI" #+amiga "AFFI" #+screen "SCREEN"))
 
 ;; Unlock the specified packages, execute the BODY, then lock them again.
 (defmacro without-package-lock (packages &body body)
@@ -148,11 +126,11 @@
     `(let ((,locked-packages
             (remove-if-not #'package-lock
                            (or ',packages *system-package-list*))))
-      (unwind-protect (progn (setf (package-lock ,locked-packages) nil)
-                             ,@body)
-        (setf (package-lock ,locked-packages) t)))))
+       (unwind-protect (progn (setf (package-lock ,locked-packages) nil)
+                              ,@body)
+         (setf (package-lock ,locked-packages) t)))))
 
-;;; Modulverwaltung (Kapitel 11.8), CLTL S. 188
+;;; module administration (Chapter 11.8), CLTL p. 188
 
 (defvar *modules* nil)
 
@@ -181,9 +159,9 @@
       (if (atom pathname) (load pathname) (mapcar #'load pathname)))))
 
 
-;;; Konstanten für Zahlen (Kapitel 12)
+;;; integer constants (Chapter 12)
 
-; vgl. File INTLOG.TXT
+;; see file INTLOG.TXT
 (defconstant boole-clr 0)
 (defconstant boole-set 15)
 (defconstant boole-1 10)
@@ -201,10 +179,10 @@
 (defconstant boole-orc1 13)
 (defconstant boole-orc2 11)
 
-; Zum Wiedereinlesen von BYTEs:
+;; for input of BYTEs:
 (defun make-byte (&key size position) (byte size position))
 
-; X3J13 vote <79>
+;; X3J13 vote <79>
 (defconstant least-positive-normalized-short-float least-positive-short-float)
 (defconstant least-negative-normalized-short-float least-negative-short-float)
 (defconstant least-positive-normalized-single-float least-positive-single-float)
@@ -218,14 +196,14 @@
     least-positive-normalized-single-float
     least-negative-normalized-single-float
     least-positive-normalized-double-float
-    least-negative-normalized-double-float
-)  )
+    least-negative-normalized-double-float))
 
+;;; functions for sequences (Chapter 14)
 
-;;; Funktionen für Sequences (Kapitel 14)
-
-(defmacro doseq ((var seqform &optional resultform) &body body &environment env)
-  (multiple-value-bind (body-rest declarations) (system::parse-body body nil env)
+(defmacro doseq ((var seqform &optional resultform) &body body
+                 &environment env)
+  (multiple-value-bind (body-rest declarations)
+      (system::parse-body body nil env)
     (let ((seqvar (gensym)))
       `(BLOCK NIL
          (LET ((,seqvar ,seqform))
@@ -234,23 +212,18 @@
              (MAP NIL
                   #'(LAMBDA (,var)
                       ,@(if declarations `((DECLARE ,@declarations)) '())
-                      (TAGBODY ,@body-rest)
-                    )
-                  ,seqvar
-             )
-             ,resultform
-       ) ) )
-) ) )
+                      (TAGBODY ,@body-rest))
+                  ,seqvar)
+             ,resultform))))))
 
 
-;;; Funktionen für Listen (Kapitel 15)
+;;; functions for lists (Chapter 15)
 
-; Hilfsversion von MEMBER, die das :KEY-Argument auch auf item anwendet:
+;; Auxiliary version of MEMBER, which applies the :KEY argument also to items
 (defun sys::member1 (item list &rest rest &key test test-not key)
   (declare (ignore test test-not))
   (unless key (setq key #'identity))
-  (apply #'member (funcall key item) list rest)
-)
+  (apply #'member (funcall key item) list rest))
 
 (defun union (list1 list2 &rest rest &key test test-not key)
   (declare (ignore test test-not key))
@@ -258,17 +231,13 @@
   (cond ((endp list1) list2)
         ((apply #'sys::member1 (car list1) list2 rest)
          (apply #'union (cdr list1) list2 rest))
-        (t (cons (car list1) (apply #'union (cdr list1) list2 rest)))
-  )
+        (t (cons (car list1) (apply #'union (cdr list1) list2 rest))))
   |# ; iterative
   (let ((list1-filtered '()))
     (dolist (item list1)
       (unless (apply #'sys::member1 item list2 rest)
-        (setq list1-filtered (cons item list1-filtered))
-    ) )
-    (nreconc list1-filtered list2)
-  )
-)
+        (setq list1-filtered (cons item list1-filtered))))
+    (nreconc list1-filtered list2)))
 
 (defun nunion (list1 list2 &rest rest &key test test-not key)
   (declare (ignore test test-not key))
@@ -276,19 +245,15 @@
   (cond ((endp list1) list2)
         ((apply #'sys::member1 (car list1) list2 rest)
          (apply #'nunion (cdr list1) list2 rest))
-        (t (rplacd list1 (apply #'nunion (cdr list1) list2 rest)))
-  )
+        (t (rplacd list1 (apply #'nunion (cdr list1) list2 rest))))
   |# ; iterative
   (let ((first nil) (last nil))
     (do ((l list1 (cdr l)))
         ((endp l))
       (unless (apply #'sys::member1 (car l) list2 rest)
         (if last (rplacd last l) (setq first l))
-        (setq last l)
-    ) )
-    (if last (progn (rplacd last list2) first) list2)
-  )
-)
+        (setq last l)))
+    (if last (progn (rplacd last list2) first) list2)))
 
 (defun intersection (list1 list2 &rest rest &key test test-not key)
   (declare (ignore test test-not key))
@@ -297,37 +262,29 @@
         ((apply #'sys::member1 (car list1) list2 rest)
          (cons (car list1)
                (apply #'intersection (cdr list1) list2 rest)))
-        (t (apply #'intersection (cdr list1) list2 rest))
-  )
+        (t (apply #'intersection (cdr list1) list2 rest)))
   |# ; iterative
   (let ((list1-filtered '()))
     (dolist (item list1)
       (when (apply #'sys::member1 item list2 rest)
-        (setq list1-filtered (cons item list1-filtered))
-    ) )
-    (list-nreverse list1-filtered)
-  )
-)
+        (setq list1-filtered (cons item list1-filtered))))
+    (list-nreverse list1-filtered)))
 
 (defun nintersection (list1 list2 &rest rest &key test test-not key)
   (declare (ignore test test-not key))
   #| ; recursive (not suitable for long lists):
   (cond ((endp list1) nil)
         ((apply #'sys::member1 (car list1) list2 rest)
-         (rplacd list1 (apply #'nintersection (cdr list1) list2 rest)) )
-        (t (apply #'nintersection (cdr list1) list2 rest))
-  )
+         (rplacd list1 (apply #'nintersection (cdr list1) list2 rest)))
+        (t (apply #'nintersection (cdr list1) list2 rest)))
   |# ; iterative
   (let ((first nil) (last nil))
     (do ((l list1 (cdr l)))
         ((endp l))
       (when (apply #'sys::member1 (car l) list2 rest)
         (if last (rplacd last l) (setq first l))
-        (setq last l)
-    ) )
-    (if last (progn (rplacd last nil) first) nil)
-  )
-)
+        (setq last l)))
+    (if last (progn (rplacd last nil) first) nil)))
 
 (defun set-difference (list1 list2 &rest rest &key test test-not key)
   (declare (ignore test test-not key))
@@ -335,61 +292,60 @@
   (cond ((endp list1) nil)
         ((not (apply #'sys::member1 (car list1) list2 rest))
          (cons (car list1)
-               (apply #'set-difference (cdr list1) list2 rest)
-        ))
-        (t (apply #'set-difference (cdr list1) list2 rest))
-  )
+               (apply #'set-difference (cdr list1) list2 rest)))
+        (t (apply #'set-difference (cdr list1) list2 rest)))
   |# ; iterative
   (let ((list1-filtered '()))
     (dolist (item list1)
       (unless (apply #'sys::member1 item list2 rest)
-        (setq list1-filtered (cons item list1-filtered))
-    ) )
-    (list-nreverse list1-filtered)
-  )
-)
+        (setq list1-filtered (cons item list1-filtered))))
+    (list-nreverse list1-filtered)))
 
 (defun nset-difference (list1 list2 &rest rest &key test test-not key)
   (declare (ignore test test-not key))
   #| ; recursive (not suitable for long lists):
   (cond ((endp list1) nil)
         ((not (apply #'sys::member1 (car list1) list2 rest))
-         (rplacd list1 (apply #'nset-difference (cdr list1) list2 rest)) )
-        (t (apply #'nset-difference (cdr list1) list2 rest))
-  )
+         (rplacd list1 (apply #'nset-difference (cdr list1) list2 rest)))
+        (t (apply #'nset-difference (cdr list1) list2 rest)))
   |# ; iterative
   (let ((first nil) (last nil))
     (do ((l list1 (cdr l)))
         ((endp l))
       (unless (apply #'sys::member1 (car l) list2 rest)
         (if last (rplacd last l) (setq first l))
-        (setq last l)
-    ) )
-    (if last (progn (rplacd last nil) first) nil)
-  )
-)
+        (setq last l)))
+    (if last (progn (rplacd last nil) first) nil)))
 
+(macrolet ((rev-arg (f) `(lambda (x y) (funcall ,f y x))))
+  ;; this is done so that the elements of LIST1 and LIST2 are passed
+  ;; to TEST and TEST-NOT always in the correct order
 (defun set-exclusive-or (list1 list2 &rest rest &key test test-not key)
-  (declare (ignore test test-not key))
+  (declare (ignore key))
   (append (apply #'set-difference list1 list2 rest)
-          (apply #'set-difference list2 list1 rest)
-) )
+          (apply #'set-difference list2 list1
+                 (cond (test (list* :test (rev-arg test) rest))
+                       (test-not (list* :test-not (rev-arg test-not) rest))
+                       (rest)))))
 
 (defun nset-exclusive-or (list1 list2 &rest rest &key test test-not key)
-  (declare (ignore test test-not key))
+  (declare (ignore key))
   (nconc (apply #'set-difference list1 list2 rest)
-         (apply #'nset-difference list2 list1 rest)
-) )
+         (apply #'nset-difference list2 list1
+                (cond (test (list* :test (rev-arg test) rest))
+                      (test-not (list* :test-not (rev-arg test-not) rest))
+                      (rest)))))
+
+) ; macrolet rev-arg
 
 (defun subsetp (list1 list2 &rest rest &key test test-not key)
   (declare (ignore test test-not key))
   (do ((l list1 (cdr l)))
       ((endp l) t)
-    (if (not (apply #'sys::member1 (car l) list2 rest)) (return nil))
-) )
+    (if (not (apply #'sys::member1 (car l) list2 rest)) (return nil))))
 
-; Wie SUBST-IF, nur dass das Ersatz-Element durch eine Funktion gegeben wird
-; und nicht konstant sein muss.
+;; Like SUBST-IF, only that the substitution element is
+;; given by a function and must not be a constant
 (defun subst-if-then (newfun testfun tree &key (key #'identity))
   (labels ((subst (tree)
              (if (funcall testfun (funcall key tree))
@@ -399,18 +355,17 @@
                         (newcar (subst car)) (newcdr (subst cdr)))
                    (if (and (eq car newcar) (eq cdr newcdr))
                      tree
-                     (cons newcar newcdr)
-                 ) )
-                 tree
-          )) ) )
-    (subst tree)
-) )
+                     (cons newcar newcdr)))
+                 tree))))
+    (subst tree)))
 
 
-;;; Funktionen für Hash-Tabellen (Kapitel 16)
+;;; functions for hash tables (Chapter 16)
 
-(defmacro dohash ((keyvar valuevar HTform &optional resultform) &body body &environment env)
-  (multiple-value-bind (body-rest declarations) (system::parse-body body nil env)
+(defmacro dohash ((keyvar valuevar HTform &optional resultform) &body body
+                  &environment env)
+  (multiple-value-bind (body-rest declarations)
+      (system::parse-body body nil env)
     (let ((HTvar (gensym)))
       `(BLOCK NIL
          (LET ((,HTvar ,HTform))
@@ -419,31 +374,24 @@
              (MAPHASH
                #'(LAMBDA (,keyvar ,valuevar)
                    ,@(if declarations `((DECLARE ,@declarations)) '())
-                   (TAGBODY ,@body-rest)
-                 )
-               ,HTvar
-             )
-             ,resultform
-       ) ) )
-) ) )
+                   (TAGBODY ,@body-rest))
+               ,HTvar)
+             ,resultform))))))
 
 
-;;; Funktionen für Strings (Kapitel 18)
+;;; functions for strings (Chapter 18)
 
 (defun string-trim (character-bag string)
-  (sys::string-both-trim character-bag character-bag string)
-)
+  (sys::string-both-trim character-bag character-bag string))
 
 (defun string-left-trim (character-bag string)
-  (sys::string-both-trim character-bag nil string)
-)
+  (sys::string-both-trim character-bag nil string))
 
 (defun string-right-trim (character-bag string)
-  (sys::string-both-trim nil character-bag string)
-)
+  (sys::string-both-trim nil character-bag string))
 
 
-;;; Functions for pathnames (Chapter 23.1.5)
+;;; functions for pathnames (Chapter 23.1.5)
 #+LOGICAL-PATHNAMES
 (export '(custom::*load-logical-pathname-translations-database*) "CUSTOM")
 #+LOGICAL-PATHNAMES
@@ -460,13 +408,14 @@
   (defun set-logical-pathname-translations (host translations)
     (setq host (string-upcase host))
     (puthash host *logical-pathname-translations* ; :test #'equal !
-             (let ((host-pathname (make-logical-pathname :host host)))
-               (mapcar #'(lambda (rule)
-                           (cons (merge-pathnames
-                                  (logical-pathname (first rule))
-                                  host-pathname 'NIL)
-                                 (rest rule)))
-                       translations))))
+             (mapcar #'(lambda (rule)
+                         (cons (parse-namestring
+                                (first rule) nil
+                                (make-logical-pathname
+                                 :host host :name :wild :type :wild
+                                 :version :wild))
+                               (rest rule)))
+                     translations)))
   ;; load many hosts from a file, AllegroCL-style
   (defun load-lpt-many (file host)
     (with-open-file (fi file :if-does-not-exist nil)
@@ -539,13 +488,12 @@
                               t)))))))))
       (error (TEXT "No translations for logical host ~S found") host)))
   (set-logical-pathname-translations "SYS"
-    '((";*.LISP" "*.lisp") ("*.*" "*.*") ("*" "*")))
-)
+    '((";*.LISP" "*.lisp") ("*" "*"))))
 
 
-;;; Funktionen für Zeit (Kapitel 25.4.1)
+;;; functions for time (Chapter 25.4.1)
 
-; Hilfsfunktion für Macro TIME
+;; help function for macro TIME
 (defun %time (new-real1 new-real2 new-run1 new-run2 new-gc1 new-gc2
               new-space1 new-space2 new-gccount
               old-real1 old-real2 old-run1 old-run2 old-gc1 old-gc2
@@ -580,11 +528,9 @@
         (write-string "GC: " stream) (write GC-Count :stream stream)
         (write-string ", GC time: " stream)
         (write (float (/ GC-Time internal-time-units-per-second)) :stream stream)
-        (write-string " sec." stream)
-      )
-) ) )
+        (write-string " sec." stream)))))
 
-; (sleep seconds) macht seconds Sekunden Pause. CLTL S. 447
+;; (sleep N) pause for N seconds. CLTL p. 447
 (defun sleep (time)
   (if (and (realp time) (not (minusp time)))
     (progn
@@ -594,31 +540,25 @@
         ; Mehr als 248 bzw. 994 bzw. 497 Tage? (Denn sys::%sleep akzeptiert nur
         ; Argumente < 2^32, bei #+OS/2 sogar nur Argumente < 2^31.)
         (loop ; ja -> Endlosschleife
-          (sys::%sleep '#,(* 86400 internal-time-units-per-second))
-        )
-        (sys::%sleep (round (* time internal-time-units-per-second)))
-      )
+          (sys::%sleep '#,(* 86400 internal-time-units-per-second)))
+        (sys::%sleep (round (* time internal-time-units-per-second))))
       #+UNIX ; SLEEP_2
       (if (> time 16700000) ; mehr als 193 Tage?
         (loop (sys::%sleep 86400 0)) ; ja -> Endlosschleife
         (multiple-value-bind (seconds rest) (floor time)
-          (sys::%sleep seconds (round (* rest 1000000)))
-      ) )
+          (sys::%sleep seconds (round (* rest 1000000)))))
       #+WIN32 ; SLEEP_2
       (if (> time 4250000) ; mehr als 49 Tage?
         (loop (sys::%sleep 86400 0)) ; ja -> Endlosschleife
         (multiple-value-bind (seconds rest) (floor time)
-          (sys::%sleep seconds (round (* rest 1000)))
-      ) )
-    )
+          (sys::%sleep seconds (round (* rest 1000))))))
     (error-of-type 'type-error
       :datum time :expected-type '(REAL 0 *)
       (TEXT "~S: argument ~S should be a nonnegative number")
-      'sleep time
-) ) )
+      'sleep time)))
 
 
-;; Funktionen für Zeit-Umrechnung und Zeitzonen (CLTL Kapitel 25.4.1)
+;; functions for Zeit-Umrechnung und Zeitzonen (CLTL Chapter 25.4.1)
 ;; Version 2, beinhaltet mehr Mathematik und basiert auf März-Jahren
 
 ; Ein März-Jahr sei die Periode vom 1.3. bis 28/29.2.
@@ -640,8 +580,7 @@
 ; und    UTag(1899) = -306
 ; gelten.
 (defun UTag (Jahr)
-  (+ (* 365 Jahr) (floor Jahr 4) (- (floor Jahr 100)) (floor Jahr 400) -693901)
-)
+  (+ (* 365 Jahr) (floor Jahr 4) (- (floor Jahr 100)) (floor Jahr 400) -693901))
 
 ; Näherungwert:
 ; 365+1/4-1/100+1/400 = 365.2425 = 146097/400 .
@@ -667,10 +606,8 @@
     ; meist richtig und jedenfalls nicht zu klein und um höchstens 1 zu groß.
     (when (< UTTag Jahresanfang) ; zu groß?
       (decf Jahr)
-      (setq Jahresanfang (UTag Jahr))
-    )
-    (values Jahr (- UTTag Jahresanfang))
-) )
+      (setq Jahresanfang (UTag Jahr)))
+    (values Jahr (- UTTag Jahresanfang))))
 
 ; Bei vielen Betriebssystemen (nicht bei UNIX, WIN32) muss die Zeitzone beim
 ; Installieren in timezone.lisp eingetragen werden. Hier stehen nur
@@ -686,8 +623,7 @@
 ; Funktion, die feststellt, ob bei gegebenem März-Jahr und Tag und Stunde
 ; Sommerzeit gilt.
 (defvar *default-dst-check* ; Default: Sommerzeit nicht explizit bekannt
-  #'(lambda (Jahr Jahrtag Stunde) (declare (ignore Jahr Jahrtag Stunde)) nil)
-)
+  #'(lambda (Jahr Jahrtag Stunde) (declare (ignore Jahr Jahrtag Stunde)) nil))
 
 ; andere Abbildung  Jahrtag -> Monat  für decode-universal-time:
 ; Seien Monat und Jahrtag auf den 1. März bezogen
@@ -723,11 +659,10 @@
 ; Dies ist allerdings langsamer als ein Tabellenzugriff.
 
 (macrolet ((Monat->Jahrtag (Monat) ; 0 <= Monat < 12, 0=März,...,11=Februar
-             `(svref '#(0 31 61 92 122 153 184 214 245 275 306 337) ,Monat)
-          ))
+             `(svref '#(0 31 61 92 122 153 184 214 245 275 306 337) ,Monat)))
 
 ; (encode-universal-time second minute hour date month year [time-zone]),
-; CLTL S. 446
+; CLTL p. 446
 (defun encode-universal-time
               (Sekunde Minute Stunde Tag Monat Jahr &optional (Zeitzone nil)
                &aux Monat3 Jahr3 Jahrtag UTTag)
@@ -737,10 +672,8 @@
                         (multiple-value-bind (i1 i2 i3 i4 i5 Jahrjetzt) (get-decoded-time)
                           (declare (ignore i1 i2 i3 i4 i5))
                           (setq Jahr
-                            (+ Jahr (* 100 (ceiling (- Jahrjetzt Jahr 50) 100)))
-                      ) ) )
-                      (<= 1900 Jahr)
-               )    )
+                            (+ Jahr (* 100 (ceiling (- Jahrjetzt Jahr 50) 100))))))
+                      (<= 1900 Jahr)))
                (and (integerp Monat) (<= 1 Monat 12))
                (progn
                  (if (< Monat 3)
@@ -753,8 +686,7 @@
                         (setq UTTag (+ Jahrtag (UTag Jahr3)))
                         (and (if (not (eql Monat3 11))
                                (< Jahrtag (Monat->Jahrtag (1+ Monat3)))
-                               (< UTTag (UTag (1+ Jahr3)))
-                             )
+                               (< UTTag (UTag (1+ Jahr3))))
                              (and (integerp Stunde) (<= 0 Stunde 23))
                              (and (integerp Minute) (<= 0 Minute 59))
                              (and (integerp Sekunde) (<= 0 Sekunde 59))
@@ -763,29 +695,22 @@
                                       (setq Zeitzone
                                         #-(or UNIX WIN32)
                                         (- *default-time-zone*
-                                           (if (funcall *default-dst-check* Jahr3 Jahrtag Stunde) 1 0)
-                                        )
+                                           (if (funcall *default-dst-check* Jahr3 Jahrtag Stunde) 1 0))
                                         #+(or UNIX WIN32)
-                                        (default-time-zone (+ (* 24 UTTag) Stunde))
-                                    ) )
+                                        (default-time-zone (+ (* 24 UTTag) Stunde))))
                                     (when (floatp Zeitzone) (setq Zeitzone (rational Zeitzone)))
                                     (or (integerp Zeitzone)
-                                        (and (rationalp Zeitzone) (integerp (* 3600 Zeitzone)))
-                                  ) )
-                                  (<= -13 Zeitzone 12)
-          )    ) )    ) )    )
+                                        (and (rationalp Zeitzone) (integerp (* 3600 Zeitzone)))))
+                                  (<= -13 Zeitzone 12)))))))
     (error-of-type 'error
       (TEXT "incorrect date: ~S.~S.~S, ~Sh~Sm~Ss, time zone ~S")
-      Tag Monat Jahr Stunde Minute Sekunde Zeitzone
-  ) )
+      Tag Monat Jahr Stunde Minute Sekunde Zeitzone))
   (+ Sekunde
      (* 60 (+ Minute
               (* 60 (+ Stunde Zeitzone
-                       (* 24 UTTag)
-  )  )     )  )     )
-)
+                       (* 24 UTTag)))))))
 
-; (decode-universal-time universal-time [time-zone]), CLTL S. 445
+; (decode-universal-time universal-time [time-zone]), CLTL p. 445
 (defun decode-universal-time (UT &optional (time-zone nil)
                               &aux Sommerzeit Zeitzone)
   (if time-zone
@@ -795,16 +720,12 @@
           Sommerzeit (let ((UT (- UT (round (* 3600 time-zone)))))
                        (multiple-value-bind (UTTag Stunde) (floor (floor UT 3600) 24)
                          (multiple-value-bind (Jahr Jahrtag) (Jahr&Tag UTTag)
-                           (funcall *default-dst-check* Jahr Jahrtag Stunde)
-                     ) ) )
-          Zeitzone (- time-zone (if Sommerzeit 1 0))
-    )
+                           (funcall *default-dst-check* Jahr Jahrtag Stunde))))
+          Zeitzone (- time-zone (if Sommerzeit 1 0)))
     #+(or UNIX WIN32)
     (progn
       (multiple-value-setq (Zeitzone Sommerzeit) (default-time-zone (floor UT 3600)))
-      (setq time-zone (+ Zeitzone (if Sommerzeit 1 0)))
-    )
-  )
+      (setq time-zone (+ Zeitzone (if Sommerzeit 1 0)))))
   ; time-zone = Zeitzone ohne Sommerzeitberücksichtigung,
   ; Zeitzone = Zeitzone mit Sommerzeitberücksichtigung.
   (let ((UTSekunden (- UT (round (* 3600 Zeitzone)))))
@@ -816,18 +737,13 @@
                    (Tag (1+ (- Jahrtag (Monat->Jahrtag Monat)))))
               (if (< Monat 10) ; Monat März..Dezember?
                 (setq Monat (+ Monat 3)) ; Monat 3..12
-                (setq Monat (- Monat 9) Jahr (+ Jahr 1)) ; Monat 1..2
-              )
+                (setq Monat (- Monat 9) Jahr (+ Jahr 1))) ; Monat 1..2
               (values Sekunde Minute Stunde Tag Monat Jahr (mod UTTage 7)
-                      Sommerzeit time-zone
-) ) ) ) ) ) ) )
+                      Sommerzeit time-zone))))))))) ; Ende von macrolet
 
-) ; Ende von macrolet
-
-; (get-decoded-time), CLTL S. 445
+; (get-decoded-time), CLTL p. 445
 (defun get-decoded-time ()
-  (decode-universal-time (get-universal-time))
-)
+  (decode-universal-time (get-universal-time)))
 
 
 ;;; Verschiedenes
@@ -838,13 +754,12 @@
   (let ((str (string-concat (string obj1) (string obj2))))
     (if (and (plusp (length str)) (eql (char str 0) #\:))
       (intern (subseq str 1) *keyword-package*)
-      (intern str)
-) ) )
+      (intern str))))
 
-; *ERROR-HANDLER* should be NIL or a function which accepts the following
-; arguments:
-; - NIL (in case of ERROR) or a continue-format-string (in case of CERROR),
-; - error-format-string,
-; - more argument list for these two format strings,
-; and which may return only if the first argument is /= NIL.
+;; *ERROR-HANDLER* should be NIL or a function which accepts the following
+;; arguments:
+;;  - NIL (in case of ERROR) or a continue-format-string (in case of CERROR),
+;;  - error-format-string,
+;;  - more argument list for these two format strings,
+;;  and which may return only if the first argument is /= NIL.
 (defvar *error-handler* nil)
