@@ -126,18 +126,6 @@ to print the corresponding values, or T for all of them.")
     (setf (fill-pointer buffer) 0)
     (clear-output target-stream)))
 
-;; Returns the length of the list, or nil if circular.
-;; The second value is the last atom (i.e., `dotted-p').
-(defun list-length-dotted (obj)
-  ;; cf. function list-length in CLtL p. 265
-  (do ((nn 0 (+ nn 2))
-       (fast obj (cddr fast))
-       (slow obj (cdr slow)))
-      (nil)
-    (when (atom fast) (return (values nn fast)))
-    (when (atom (cdr fast)) (return (values (1+ nn) (cdr fast))))
-    (when (eq (cdr fast) slow) (return nil))))
-
 ; List of objects which have been described during the current top-level call.
 (defvar *describe-done*)
 
@@ -161,10 +149,7 @@ to print the corresponding values, or T for all of them.")
 (clos:defgeneric describe-object (obj stream)
   (:method ((obj t) (stream stream))
     (ecase (type-of obj)
-      #+DIR-KEY
-      (LDAP::DIR-KEY
-       (format stream (TEXT "a directory access key")))
-      #+(or UNIX AMIGA FFI DIR-KEY)
+      #+(or UNIX AMIGA FFI WIN32)
       (EXT::FOREIGN-POINTER
        (format stream (TEXT "a foreign pointer")))
       #+FFI
@@ -173,7 +158,7 @@ to print the corresponding values, or T for all of them.")
       #+FFI
       (FFI::FOREIGN-VARIABLE
        (format stream (TEXT "a foreign variable of foreign type ~S.")
-               (deparse-c-type (sys::%record-ref obj 3))))
+               (ffi::deparse-c-type (sys::%record-ref obj 3))))
       (BYTE
        (format stream (TEXT "a byte specifier, denoting the ~S bits starting at bit position ~S of an integer.")
                (byte-size obj) (byte-position obj)))
@@ -183,8 +168,12 @@ to print the corresponding values, or T for all of them.")
        (format stream (TEXT "a load-time evaluation promise.")))
       (EXT:SYMBOL-MACRO
        (format stream (TEXT "a symbol macro handler.")))
-      (EXT:MACRO
-       (format stream (TEXT "a macro expander.")))
+      (SYS::MACRO
+       (format stream (TEXT "a macro expander."))
+       (format stream (TEXT "~%For more information, evaluate ~{~S~^ or ~}.")
+               `((DISASSEMBLE (MACRO-FUNCTION
+                               ',(sys::closure-name
+                                  (sys::macro-expander obj)))))))
       (EXT:FUNCTION-MACRO
        (format stream (TEXT "a function with alternative macro expander.")))
       (EXT:ENCODING
@@ -422,9 +411,10 @@ to print the corresponding values, or T for all of them.")
               (simple-array-p obj) rank)
       (when (eql rank 1)
         (format stream (TEXT " (vector)")))
-      (unless (eq eltype 'T)
-        (format stream (TEXT " of ~(~A~)s")
-                eltype))
+      (case eltype
+        ((T))
+        ((NIL) (write-string (TEXT " with no storage") stream))
+        (OTHERWISE (format stream (TEXT " of ~As") eltype)))
       (when (adjustable-array-p obj)
         (format stream (TEXT ", adjustable")))
       (when (plusp rank)
@@ -433,16 +423,22 @@ to print the corresponding values, or T for all of them.")
         (when (array-has-fill-pointer-p obj)
           (format stream (TEXT " and current length (fill-pointer) ~S")
                   (fill-pointer obj))))
+      (when (stringp obj)
+        (multiple-value-bind (bits ro-p realloc) (sys::string-info obj)
+          (write-string (TEXT " (a ") stream)
+          (when ro-p (write-string (TEXT "immutable ") stream))
+          (when realloc (write-string (TEXT "reallocated ") stream))
+          (format stream (TEXT "~S-bit string)") bits)))
       (format stream (TEXT "."))))
   (:method ((obj function) (stream stream))
     (ecase (type-of obj)
       #+FFI
       (FFI::FOREIGN-FUNCTION
        (format stream (TEXT "a foreign function of foreign type ~S.")
-               (deparse-c-type (vector 'ffi::c-function
-                                       (sys::%record-ref obj 2)
-                                       (sys::%record-ref obj 3)
-                                       (sys::%record-ref obj 4)))))
+               (ffi::deparse-c-type (vector 'ffi::c-function
+                                            (sys::%record-ref obj 2)
+                                            (sys::%record-ref obj 3)
+                                            (sys::%record-ref obj 4)))))
       (COMPILED-FUNCTION ; SUBR
        (format stream (TEXT "a built-in system function."))
        (multiple-value-bind (name req opt rest-p keywords other-keys)
@@ -459,14 +455,14 @@ to print the corresponding values, or T for all of them.")
              (sys::signature obj)
            (sys::describe-signature stream req opt rest-p key-p keywords
                                     other-keys-p)
-           (format stream (TEXT "~%For more information, evaluate ~{~S~^ or ~}.")
-                   (let* ((name (sys::closure-name obj))
-                          (funform
-                            (if (and (symbolp name) (macro-function name))
-                              `(MACRO-FUNCTION ',name)
-                              `(FUNCTION ,name)
-                         )) )
-                     `((DISASSEMBLE ,funform)))))
+           (let* ((name (sys::closure-name obj))
+                  (funform (cond ((and (symbolp name) (macro-function name))
+                                  `(MACRO-FUNCTION ',name))
+                                 ((fboundp name) `(FUNCTION ,name)))))
+             (when funform
+               (format stream
+                       (TEXT "~%For more information, evaluate ~{~S~^ or ~}.")
+                       `((DISASSEMBLE ,funform))))))
          (let ((doc (sys::%record-ref obj 2)))
            (format stream (TEXT "~%argument list: ~:S")
                    (car (sys::%record-ref obj 1)))
@@ -478,10 +474,15 @@ to print the corresponding values, or T for all of them.")
                     obj (or *print-right-margin* sys::*prin-linelength*))))
     (if (memq obj *describe-done*)
       (format stream (TEXT "~&~%~A [see above]") objstring)
-      (progn
+      (let ((doc (gethash obj *documentation*)))
         (push obj *describe-done*)
         (format stream (TEXT "~&~%~A is ") objstring)
-        (describe-object obj stream))))
+        (describe-object obj stream)
+        (when doc
+          (format stream (TEXT "~&Documentation:"))
+          (do ((tail doc (cddr tail)))
+              ((endp tail))
+            (format stream "~&~s:~%~s" (car tail) (cadr tail)))))))
   (finish-output stream))
 
 (defun describe (obj &optional stream)
@@ -541,7 +542,7 @@ to print the corresponding values, or T for all of them.")
 ;; Return the line associated with a Unicode code in the Unicode data file.
 ;; Returns a simple-string or nil. Also used by the CHAR-NANE function.
 (defun unicode-attributes-line (code)
-  (with-open-file (f (clisp-data-file "UnicodeData.txt")
+  (with-open-file (f (clisp-data-file "UnicodeDataFull.txt")
                      :direction :input
                      :element-type 'character
                      :external-format 'charset:ascii)
@@ -556,12 +557,25 @@ to print the corresponding values, or T for all of them.")
                      (c3 (read-char f nil nil))
                      (c4 (read-char f nil nil)))
                  (if (and c1 c2 c3 c4)
-                   (parse-integer (coerce (list c1 c2 c3 c4) 'string)
-                                  :radix 16
-                   )
-                   #x10000
+                   (let ((c5 (read-char f nil nil)))
+                     (if (and c5 (digit-char-p c5 16))
+                       (let ((c6 (read-char f nil nil)))
+                         (if (and c6 (digit-char-p c6 16))
+                           (parse-integer
+                             (coerce (list c1 c2 c3 c4 c5 c6) 'string)
+                             :radix 16
+                           )
+                           (parse-integer
+                             (coerce (list c1 c2 c3 c4 c5) 'string)
+                             :radix 16
+                       ) ) )
+                       (parse-integer
+                         (coerce (list c1 c2 c3 c4) 'string)
+                         :radix 16
+                   ) ) )
+                   #x1000000
                ) )
-               #x10000
+               #x1000000
           )) )
       (let* ((blocksize 1024)
              (f-size (progn (file-position f :end) (file-position f)))
@@ -585,12 +599,14 @@ to print the corresponding values, or T for all of them.")
         (loop
           (let ((line (read-line f nil nil)))
             (unless line (return nil))
-            (let ((c (parse-integer line :end 4 :radix 16)))
+            (let* ((four (position #\; line)) ; 4 or 5 or 6
+                   (c (parse-integer line :end four :radix 16)))
               ; Treat the range start/end lines specially.
-              (when (eql (char line 4) #\;)
-                (when (eql (char line 5) #\<)
-                  (let* ((piece1 (subseq line 5
-                                              (or (position #\; line :start 5)
+              (when (eql (char line four) #\;)
+                (when (eql (char line (+ four 1)) #\<)
+                  (let* ((piece1 (subseq line (+ four 1)
+                                              (or (position #\; line
+                                                            :start (+ four 1))
                                                   (length line))))
                          (n (length piece1)))
                     (cond ((and (= c code)
@@ -600,8 +616,8 @@ to print the corresponding values, or T for all of them.")
                            (return
                              (concatenate 'string
                                (format nil "~4,'0X" code)
-                               (subseq line 4 (+ 5 n -8))
-                               (subseq line (+ 5 n -1))
+                               (subseq line four (+ four n -8 1))
+                               (subseq line (+ four n))
                           )) )
                           ((and (>= c code)
                                 (>= n 7)
@@ -610,8 +626,8 @@ to print the corresponding values, or T for all of them.")
                            (return
                              (concatenate 'string
                                (format nil "~4,'0X" code)
-                               (subseq line 4 (+ 5 n -7))
-                               (subseq line (+ 5 n -1))
+                               (subseq line four (+ four n -7 1))
+                               (subseq line (+ four n))
                           )) )
               ) ) ) )
               (if (= c code) (return line))
@@ -638,27 +654,21 @@ to print the corresponding values, or T for all of them.")
 ;; 14. Title case equivalent mapping.
 (defun unicode-attributes (ch)
   (let ((line (unicode-attributes-line (char-code ch))))
-    (if line
+    (when line
       (let ((pieces
-              (loop for pos = 0 then (1+ semicolon-pos)
-                    for semicolon-pos = (position #\; line :start pos)
-                    do collect (subseq line pos (or semicolon-pos (length line)))
-                    do (unless semicolon-pos (loop-finish))
-           )) )
+             (loop :for pos = 0 :then (1+ semicolon-pos)
+               :for semicolon-pos = (position #\; line :start pos)
+               :collect (subseq line pos (or semicolon-pos (length line)))
+               :do (unless semicolon-pos (loop-finish)))))
         (assert (= (parse-integer (nth 0 pieces) :radix 16) (char-code ch)))
         (values-list
-          (append
-            (mapcar #'(lambda (x) (if (equal x "") nil x))
-                    (subseq pieces 1 12)
-            )
-            (mapcar #'(lambda (x)
-                        (if (equal x "") nil (code-char (parse-integer x :radix 16)))
-                      )
-                    (subseq pieces 12 15)
-        ) ) )
-      )
-      nil
-) ) )
+         (append
+          (mapcar #'(lambda (x) (if (equal x "") nil x))
+                  (subseq pieces 1 12))
+          (mapcar #'(lambda (x)
+                      (if (equal x "") nil
+                          (code-char (parse-integer x :radix 16))))
+                  (subseq pieces 12 15))))))))
 
 ) ; #+UNICODE
 
