@@ -6,34 +6,24 @@
 ;;;----------------------------------------------------------------------------
 ;;; Funktionen zur Definition und zum Ausnutzen von places:
 ;;;----------------------------------------------------------------------------
-;; Return an un-interned symbol for SYSTEM::SETF-FUNCTION
+;; Return a symbol for SYSTEM::SETF-FUNCTION
+;; the returned symbol will be interned iff the argument is.
 (defun setf-symbol (symbol)
-  (make-symbol
-    (string-concat
-      "(SETF "
-      (let ((pack (symbol-package symbol))) (if pack (package-name pack) "#"))
-      ":"
-      (symbol-name symbol)
-      ")"
-) ) )
+  (let* ((pack (symbol-package symbol))
+         (name (string-concat "(SETF " (if pack (package-name pack) "#") ":"
+                              (symbol-name symbol) ")")))
+    (if pack
+        (intern name pack)
+        (make-symbol name))))
 ;;;----------------------------------------------------------------------------
 ;; Returns the symbol which is on the property list at SYSTEM::SETF-FUNCTION
 (defun get-setf-symbol (symbol)
   (or (get symbol 'SYSTEM::SETF-FUNCTION)
       (progn
         (when (get symbol 'SYSTEM::SETF-EXPANDER)
-          (warn (ENGLISH "The function (~S ~S) is hidden by a SETF expander.")
-                'setf symbol
-        ) )
-        (setf (get symbol 'SYSTEM::SETF-FUNCTION) (setf-symbol symbol))
-) )   )
-;;;----------------------------------------------------------------------------
-;; Mapping funname -> symbol
-(defun get-funname-symbol (funname)
-  (if (atom funname)
-    funname
-    (get-setf-symbol (second funname))
-) )
+          (warn (TEXT "The function (~S ~S) is hidden by a SETF expander.")
+                'setf symbol))
+        (setf (get symbol 'SYSTEM::SETF-FUNCTION) (setf-symbol symbol)))))
 ;;;----------------------------------------------------------------------------
 ;; Returns 5 values:
 ;;   SM1  vr  variables to bind
@@ -83,7 +73,7 @@
                             (if (keywordp argform)
                               (push argform new-access-form)
                               (error-of-type 'source-program-error
-                                (ENGLISH "The argument ~S to ~S should be a keyword.")
+                                (TEXT "The argument ~S to ~S should be a keyword.")
                                 argform (car access-form)
                             ) )
                             (let ((tempvar (gensym)))
@@ -94,11 +84,8 @@
                           (incf i)
                       ) )
                       (setq new-access-form
-                        (cons (car access-form) (nreverse new-access-form))
-                      )
-                      (let ((newval-vars
-                              (map-into (make-list (cadr plist-info))
-                                        #'gensym)))
+                        (cons (car access-form) (nreverse new-access-form)))
+                      (let ((newval-vars (gensym-list (cadr plist-info))))
                         (values
                           (nreverse tempvars)
                           (nreverse tempforms)
@@ -138,19 +125,15 @@
                 ))
         )) )
         (t (error-of-type 'source-program-error
-             (ENGLISH "Argument ~S is not a SETF place.")
-             form
-  )     )  )
-)
-;; backward compatibility
-(sys::%putd 'get-setf-method-multiple-value #'get-setf-expansion)
+             (TEXT "~S: Argument ~S is not a SETF place.")
+             'get-setf-expansion form))))
 ;;;----------------------------------------------------------------------------
 (defun get-setf-method (form &optional (env (vector nil nil)))
   (multiple-value-bind (vars vals stores store-form access-form)
       (get-setf-expansion form env)
     (unless (and (consp stores) (null (cdr stores)))
       (error-of-type 'source-program-error
-        (ENGLISH "SETF place ~S produces more than one store variable.")
+        (TEXT "SETF place ~S produces more than one store variable.")
         form
     ) )
     (values vars vals stores store-form access-form)
@@ -171,61 +154,37 @@
            (simple-use-p (third form) var)
 ) )   )
 ;;;----------------------------------------------------------------------------
-(defun documentation (symbol doctype)
-  (unless (function-name-p symbol)
-    (error-of-type 'error
-      (ENGLISH "~S: first argument ~S is illegal, not a symbol")
-      'documentation symbol
-  ) )
-  (getf (get (get-funname-symbol symbol) 'SYSTEM::DOCUMENTATION-STRINGS) doctype)
-)
-(defun SYSTEM::%SET-DOCUMENTATION (symbol doctype value)
-  (unless (function-name-p symbol)
-    (error-of-type 'error
-      (ENGLISH "~S: first argument ~S is illegal, not a symbol")
-      'documentation symbol
-  ) )
-  (setq symbol (get-funname-symbol symbol))
-  (if (null value)
-    (when (getf (get symbol 'SYSTEM::DOCUMENTATION-STRINGS) doctype)
-      (remf (get symbol 'SYSTEM::DOCUMENTATION-STRINGS) doctype)
-      nil
-    )
-    (setf (getf (get symbol 'SYSTEM::DOCUMENTATION-STRINGS) doctype) value)
-) )
-;;;----------------------------------------------------------------------------
 (defmacro push (item place &environment env)
   (multiple-value-bind (vr vl sv se ge) (get-setf-expansion place env)
     (if (simple-assignment-p se sv)
       (subst `(CONS ,item ,ge) (car sv) se)
       (let* ((bindlist (mapcar #'list vr vl))
-             (tempvars (map-into (make-list (length sv)) #'gensym))
+             (tempvars (gensym-list (length sv)))
              (ns (sublis (mapcar #'(lambda (storevar tempvar)
-                                     (cons storevar `(CONS ,storevar ,tempvar))
-                                   )
-                                 sv tempvars
-                         )
-                         se
-            ))   )
+                                     (cons storevar
+                                           `(CONS ,storevar ,tempvar)))
+                                 sv tempvars)
+                         se)))
         `(MULTIPLE-VALUE-BIND ,sv ,item
            (LET* ,bindlist
              (MULTIPLE-VALUE-BIND ,tempvars ,ge
-               ,ns
-         ) ) )
-) ) ) )
+               ,ns)))))))
 ;;;----------------------------------------------------------------------------
+(eval-when (load compile eval)
+  (defun check-accessor-name (accessfn)
+    (unless (symbolp accessfn)
+      (error-of-type 'source-program-error
+        (TEXT "The name of the accessor must be a symbol, not ~S")
+        accessfn))))
 (defmacro define-setf-expander (accessfn lambdalist &body body
                                 &environment env)
-  (unless (symbolp accessfn)
-    (error-of-type 'source-program-error
-      (ENGLISH "The name of the access function must be a symbol, not ~S")
-      accessfn
-  ) )
+  (check-accessor-name accessfn)
   (multiple-value-bind (body-rest declarations docstring)
       (system::parse-body body t env)
     (if (null body-rest) (setq body-rest '(NIL)))
     (let ((name (make-symbol (string-concat "SETF-" (symbol-name accessfn)))))
-      (multiple-value-bind (newlambdalist envvar) (remove-env-arg lambdalist name)
+      (multiple-value-bind (newlambdalist envvar)
+          (remove-env-arg lambdalist name)
         (let ((SYSTEM::%ARG-COUNT 0)
               (SYSTEM::%MIN-ARGS 0)
               (SYSTEM::%RESTP nil)
@@ -251,7 +210,7 @@
               (setq mainform
                 `(IF ,lengthtest
                    (ERROR-OF-TYPE 'PROGRAM-ERROR
-                     (ENGLISH "The SETF expander for ~S may not be called with ~S arguments.")
+                     (TEXT "The SETF expander for ~S may not be called with ~S arguments.")
                      (QUOTE ,accessfn) (1- (LENGTH SYSTEM::%LAMBDA-LIST))
                    )
                    ,mainform
@@ -264,6 +223,10 @@
                    ,@(if envvar '() '((DECLARE (IGNORE SYSTEM::ENV))))
                    ,mainform
                  )
+                 (sys::check-redefinition
+                  ',accessfn 'define-setf-expander
+                  (and (get ',accessfn 'SYSTEM::SETF-EXPANDER)
+                       "SETF expander"))
                  (SYSTEM::%PUT ',accessfn 'SYSTEM::SETF-EXPANDER
                    (CONS -5 (FUNCTION ,name))
                  )
@@ -271,15 +234,16 @@
                  ',accessfn
              ) )
 ) ) ) ) ) )
-(defmacro define-setf-method (&rest args) ; backward compatibility
-  `(DEFINE-SETF-EXPANDER ,@args)
-)
 ;;;----------------------------------------------------------------------------
 (defmacro defsetf (accessfn &rest args &environment env)
+  (check-accessor-name accessfn)
   (cond ((and (consp args) (not (listp (first args))) (symbolp (first args)))
          `(EVAL-WHEN (LOAD COMPILE EVAL)
             (LET ()
               (REMPROP ',accessfn 'SYSTEM::DEFSTRUCT-WRITER)
+              (SYS::CHECK-REDEFINITION
+               ',accessfn 'DEFSETF
+               (and (get ',accessfn 'SYSTEM::SETF-EXPANDER) "SETF expander"))
               (SYSTEM::%PUT ',accessfn 'SYSTEM::SETF-EXPANDER ',(first args))
               (SYSTEM::%SET-DOCUMENTATION ',accessfn 'SETF
                 ,(if (and (null (cddr args))
@@ -288,13 +252,11 @@
                    (second args)
                    (if (cddr args)
                      (error-of-type 'source-program-error
-                       (ENGLISH "Too many arguments to DEFSETF: ~S")
-                       (cdr args)
-                     )
+                       (TEXT "~S: Too many arguments: ~S")
+                       'defsetf (cdr args))
                      (error-of-type 'source-program-error
-                       (ENGLISH "The doc string to DEFSETF must be a string: ~S")
-                       (second args)
-                 ) ) )
+                       (TEXT "~S: The documentation string must be a string: ~S")
+                       'defsetf (second args))))
               )
               ',accessfn
           ) )
@@ -302,7 +264,7 @@
         ((and (consp args) (listp (first args)) (consp (cdr args)) (listp (second args)))
          (when (null (second args))
            (error-of-type 'source-program-error
-             (ENGLISH "Missing store variable in DEFSETF.")))
+             (TEXT "~S: Missing store variable.") 'defsetf))
          (multiple-value-bind (body-rest declarations docstring)
              (system::parse-body (cddr args) t env)
            (let* ((storevars (second args))
@@ -334,6 +296,10 @@
              `(EVAL-WHEN (LOAD COMPILE EVAL)
                 (LET ()
                   (REMPROP ',accessfn 'SYSTEM::DEFSTRUCT-WRITER)
+                  (SYS::CHECK-REDEFINITION
+                   ',accessfn 'DEFSETF
+                   (and (get ',accessfn 'SYSTEM::SETF-EXPANDER)
+                        "SETF expander"))
                   (SYSTEM::%PUT ',accessfn 'SYSTEM::SETF-EXPANDER
                     (LIST* ,arg-count ,(length storevars)
                            (FUNCTION ,(concat-pnames "SETF-" accessfn) ,setter)
@@ -343,12 +309,15 @@
               ) )
         )) )
         (t (error-of-type 'source-program-error
-             (ENGLISH "Illegal syntax in DEFSETF for ~S")
-             accessfn
-) )     )  )
+             (TEXT "(~S ~S): Illegal syntax.")
+             'defsetf accessfn))))
 ;;;----------------------------------------------------------------------------
 ;;; Definition of places:
 ;;;----------------------------------------------------------------------------
+(defsetf package-lock SYSTEM::%SET-PACKAGE-LOCK)
+(defsetf hash-table-weak-p SYSTEM::%SET-HASH-TABLE-WEAK-P)
+(defsetf weak-pointer-value SYSTEM::%SET-WEAK-POINTER-VALUE)
+;;;----------------------------------------------------------------------------
 (defsetf aref (array &rest indices) (value)
   `(SYSTEM::STORE ,array ,@indices ,value))
 ;;;----------------------------------------------------------------------------
@@ -356,9 +325,8 @@
   (let ((pointer (nthcdr index list)))
     (if (null pointer)
       (error-of-type 'error
-        (ENGLISH "(SETF (NTH ...) ...) : index ~S is too large for ~S")
-        index list
-      )
+        (TEXT "~S: index ~S is too large for ~S")
+        '(setf nth) index list)
       (rplaca pointer value)
     )
     value
@@ -440,14 +408,9 @@
                  (VALUES ,@(mapcar #'(lambda (storevar) `(CAR ,storevar)) sv))
                  ,@(devalue-form
                      (sublis (mapcar #'(lambda (storevar)
-                                         (cons storevar `(CDR ,storevar))
-                                       )
-                                     sv
-                             )
-                             se
-                   ) )
-           ) ) )
-) ) ) ) )
+                                         (cons storevar `(CDR ,storevar)))
+                                     sv)
+                             se))))))))))
 ;----------------------------------------------------------------------------
 (defmacro psetf (&whole form &rest args &environment env)
   (labels ((recurse (args)
@@ -456,66 +419,59 @@
                (declare (ignore ge))
                (when (atom (cdr args))
                  (error-of-type 'source-program-error
-                   (ENGLISH "PSETF called with an odd number of arguments: ~S")
-                   form))
+                   (TEXT "~S called with an odd number of arguments: ~S")
+                   'psetf form))
                `(LET* ,(mapcar #'list vr vl)
                   (MULTIPLE-VALUE-BIND ,sv ,(second args)
                     ,@(when (cddr args) (list (recurse (cddr args))))
-                    ,@(devalue-form se)
-                ) )
-          )) )
-    `(,@(recurse args) NIL)
-) )
+                    ,@(devalue-form se))))))
+    (when args `(,@(recurse args) NIL))))
 ;;;----------------------------------------------------------------------------
 (defmacro pushnew (item place &rest keylist &environment env)
   (multiple-value-bind (vr vl sv se ge) (get-setf-expansion place env)
     (if (simple-assignment-p se sv)
       (subst `(ADJOIN ,item ,ge ,@keylist) (car sv) se)
       (let* ((bindlist (mapcar #'list vr vl))
-             (tempvars (map-into (make-list (length sv)) #'gensym))
+             (tempvars (gensym-list (length sv)))
              (ns (sublis (mapcar #'(lambda (storevar tempvar)
-                                     (cons storevar `(ADJOIN ,storevar ,tempvar ,@keylist))
-                                   )
-                                 sv tempvars
-                         )
-                         se
-            ))   )
+                                     (cons storevar `(ADJOIN ,storevar ,tempvar
+                                                      ,@keylist)))
+                                 sv tempvars)
+                         se)))
         `(MULTIPLE-VALUE-BIND ,sv ,item
            (LET* ,bindlist
              (MULTIPLE-VALUE-BIND ,tempvars ,ge
-               ,ns
-         ) ) )
-) ) ) )
+               ,ns)))))))
 ;;;----------------------------------------------------------------------------
 (defmacro remf (place indicator &environment env)
   (multiple-value-bind (SM1 SM2 SM3 SM4 SM5) (get-setf-method place env)
     (let* ((indicatorvar (gensym))
            (bindlist
-             `(,@(mapcar #'list SM1 SM2)
-               (,(first SM3) ,SM5)
-               (,indicatorvar ,indicator)))
-           (var1 (gensym))
-           (var2 (gensym)))
+            `(,@(mapcar #'list SM1 SM2)
+              (,(first SM3) ,SM5)
+              (,indicatorvar ,indicator)))
+           (new-plist (gensym))
+           (removed-p (gensym)))
       `(LET* ,bindlist
-         (DO ((,var1 ,(first SM3) (CDDR ,var1))
-              (,var2 NIL ,var1))
-             ((ATOM ,var1) NIL)
-           (COND ((ATOM (CDR ,var1))
-                  (ERROR-OF-TYPE 'ERROR
-                    (ENGLISH "REMF: property list with an odd length")
-                 ))
-                 ((EQ (CAR ,var1) ,indicatorvar)
-                  (IF ,var2
-                    (RPLACD (CDR ,var2) (CDDR ,var1))
-                    ,(let ((newvalform `(CDDR ,(first SM3))))
-                       (if (simple-assignment-p SM4 SM3)
-                         (subst newvalform (first SM3) SM4)
-                         `(PROGN (SETQ ,(first SM3) ,newvalform) ,SM4)
-                     ) )
-                 )
-                 (RETURN T)
-      ) ) )     )
-) ) )
+         (multiple-value-bind (,new-plist ,removed-p)
+             (sys::%remf ,(first SM3) ,indicatorvar)
+           (when (and ,removed-p (atom ,new-plist))
+             ,(if (simple-assignment-p SM4 SM3)
+                  (subst new-plist (first SM3) SM4)
+                  `(PROGN (SETQ ,(first SM3) ,new-plist) ,SM4)))
+           ,removed-p)))))
+;;;----------------------------------------------------------------------------
+(export 'ext::remove-plist "EXT")
+(defun remove-plist (plist &rest keys)
+  ;; Remove the keys from the plist.
+  ;; Useful for re-using the &REST arg after removing some options.
+  (do (copy rest)
+      ((null (setq rest (nth-value 2 (get-properties plist keys))))
+       (nreconc copy plist))
+    (do () ((eq plist rest))
+      (push (pop plist) copy)
+      (push (pop plist) copy))
+    (setq plist (cddr plist))))
 ;;;----------------------------------------------------------------------------
 (defmacro rotatef (&rest args &environment env)
   (when (null args) (return-from rotatef NIL))
@@ -552,29 +508,23 @@
              (if (symbolp (second lambdalistr))
                (setq restvar (second lambdalistr))
                (error-of-type 'source-program-error
-                 (ENGLISH "In the definition of ~S: &REST variable ~S should be a symbol.")
-                 name (second lambdalistr)
-             ) )
+                 (TEXT "~S ~S: &REST variable ~S should be a symbol.")
+                 'define-modify-macro name (second lambdalistr)))
              (if (null (cddr lambdalistr))
                (return)
                (error-of-type 'source-program-error
-                 (ENGLISH "Only one variable is allowed after &REST, not ~S")
-                 lambdalistr
-            )) )
+                 (TEXT "~S(~S): Only one variable is allowed after &REST, not ~S")
+                 'define-modify-macro name lambdalistr)))
             ((or (eq next '&KEY) (eq next '&ALLOW-OTHER-KEYS) (eq next '&AUX))
              (error-of-type 'source-program-error
-               (ENGLISH "Illegal in a DEFINE-MODIFY-MACRO lambda list: ~S")
-               next
-            ))
+               (TEXT "Illegal in a ~S lambda list: ~S")
+               'define-modify-macro next))
             ((symbolp next) (push next varlist))
             ((and (listp next) (symbolp (first next)))
-             (push (first next) varlist)
-            )
+             (push (first next) varlist))
             (t (error-of-type 'source-program-error
-                 (ENGLISH "lambda list may only contain symbols and lists, not ~S")
-                 next
-            )  )
-    ) )
+                 (TEXT "~S: lambda list may only contain symbols and lists, not ~S")
+                 'define-modify-macro next))))
     (setq varlist (nreverse varlist))
     `(DEFMACRO ,name (%REFERENCE ,@lambdalist &ENVIRONMENT ENV) ,docstring
        (MULTIPLE-VALUE-BIND (DUMMIES VALS NEWVAL SETTER GETTER)
@@ -688,15 +638,12 @@
                        )
                    ))
                    (t (error-of-type 'source-program-error
-                        (ENGLISH "Illegal SETF place: ~S")
-                        (first args)
-             )     )  )
-          ))
+                        (TEXT "~S: Illegal place: ~S")
+                        'setf (first args))))))
           ((oddp argcount)
            (error-of-type 'source-program-error
-             (ENGLISH "~S called with an odd number of arguments: ~S")
-             'setf form
-          ))
+             (TEXT "~S called with an odd number of arguments: ~S")
+             'setf form))
           (t (do* ((arglist args (cddr arglist))
                    (L nil))
                   ((null arglist) `(LET () (PROGN ,@(nreverse L))))
@@ -707,8 +654,8 @@
 (defmacro shiftf (&whole form &rest args &environment env)
   (when (< (length args) 2)
     (error-of-type 'source-program-error
-      (ENGLISH "SHIFTF called with too few arguments: ~S")
-      form))
+      (TEXT "~S: too few arguments: ~S")
+      'shiftf form))
   (do* ((arglist args (cdr arglist))
         (res (list 'let* nil nil)) lf ff
         (tail (cdr res)) bindlist stores lv fv)
@@ -742,17 +689,6 @@
 ;;; Schreibt zu einem bestimmten Indicator einen Wert in eine gegebene
 ;;; Propertyliste. Wert ist NIL falls erfolgreich getan oder die neue
 ;;; (erweiterte) Propertyliste.
-(defun sys::%putf (plist indicator value)
-  (do ((plistr plist (cddr plistr)))
-      ((atom plistr) (list* indicator value plist))
-    (when (atom (cdr plistr))
-      (error-of-type 'error
-        (ENGLISH "(SETF (GETF ...) ...) : property list with an odd length")
-    ))
-    (when (eq (car plistr) indicator)
-      (rplaca (cdr plistr) value)
-      (return nil)
-) ) )
 (define-setf-expander getf (place indicator &optional default &environment env)
   (multiple-value-bind (SM1 SM2 SM3 SM4 SM5) (get-setf-method place env)
     (let* ((storevar (gensym))
@@ -778,21 +714,6 @@
       `,storeform
 ) ) )
 ;;;----------------------------------------------------------------------------
-#| ; siehe oben:
-(defun SYSTEM::%SET-DOCUMENTATION (symbol doctype value)
-  (unless (function-name-p symbol)
-    (error-of-type 'error
-      (ENGLISH "first argument ~S is illegal, not a symbol")
-      symbol
-  ) )
-  (setq symbol (get-funname-symbol symbol))
-  (if (null value)
-    (progn (remf (get symbol 'SYSTEM::DOCUMENTATION-STRINGS) doctype) nil)
-    (setf (getf (get symbol 'SYSTEM::DOCUMENTATION-STRINGS) doctype) value)
-) )
-|#
-(defsetf documentation SYSTEM::%SET-DOCUMENTATION)
-;;;----------------------------------------------------------------------------
 (defsetf fill-pointer SYSTEM::SET-FILL-POINTER)
 ;;;----------------------------------------------------------------------------
 (defsetf readtable-case SYSTEM::SET-READTABLE-CASE)
@@ -893,15 +814,14 @@
       )
     (setq fun (second fun))
     (error-of-type 'source-program-error
-      (ENGLISH "SETF APPLY is only defined for functions of the form #'symbol.")
-  ) )
+      (TEXT "~S is only defined for functions of the form #'symbol.")
+      '(setf apply)))
   (multiple-value-bind (SM1 SM2 SM3 SM4 SM5)
       (get-setf-expansion (cons fun args) env)
     (unless (eq (car (last args)) (car (last SM2)))
       (error-of-type 'source-program-error
-        (ENGLISH "APPLY on ~S is not a SETF place.")
-        fun
-    ) )
+        (TEXT "~S on ~S is not a SETF place.")
+        'apply fun))
     (let ((item (car (last SM1)))) ; 'item' steht für eine Argumentliste!
       (labels ((splice (arglist)
                  ; Würde man in (LIST . arglist) das 'item' nicht als 1 Element,
@@ -954,8 +874,8 @@
                (setq fun (second fun))
           )
     (error-of-type 'source-program-error
-      (ENGLISH "SETF FUNCALL is only defined for functions of the form #'symbol.")
-  ) )
+      (TEXT "~S is only defined for functions of the form #'symbol.")
+      '(setf funcall)))
   (get-setf-expansion (cons fun args) env)
 )
 ;;;----------------------------------------------------------------------------
@@ -1001,9 +921,8 @@
           (get-setf-expansion f-form env)
         (unless (eql (length T-SM3) (length F-SM3))
           (error-of-type 'source-program-error
-            (ENGLISH "SETF place ~S expects different numbers of values in the true and branches (~D vs. ~D values).")
-            (list 'IF condition t-form f-form) (length T-SM3) (length F-SM3)
-        ) )
+            (TEXT "SETF place ~S expects different numbers of values in the true and false branches (~D vs. ~D values).")
+            (list 'IF condition t-form f-form) (length T-SM3) (length F-SM3)))
         (values
           `(,conditionvar
             ,@T-SM1
@@ -1098,6 +1017,16 @@ Variables affected: `custom:*floating-point-contagion-ansi*',
 Invoking CLISP with `-ansi' sets this to T.
 Invoking CLISP with `-traditional' sets this to NIL.")
 
+(define-symbol-macro *current-language* (sys::current-language))
+(defsetf sys::current-language sys::set-current-language)
+(system::%set-documentation '*current-language* 'variable
+ "This symbol-macro determines the current language used for UI.")
+
+(define-symbol-macro *lib-directory* (sys::lib-directory))
+(defsetf sys::lib-directory sys::set-lib-directory)
+(system::%set-documentation '*lib-directory* 'variable
+ "This symbol-macro determines the location where CLISP finds its data files.")
+
 (define-symbol-macro *default-file-encoding*
   (system::default-file-encoding))
 (defsetf system::default-file-encoding system::set-default-file-encoding)
